@@ -2,21 +2,37 @@
 
 import abc
 import warnings
+
 import numpy as np
-import scipy.sparse as sps
 import scipy.optimize as opt
 from scipy.optimize import minimize
+
+from dipy.testing.decorators import warning_for_keywords
 from dipy.utils.optpkg import optional_package
 
 cvxpy, have_cvxpy, _ = optional_package("cvxpy", min_version="1.4.1")
 
 
 class Optimizer:
-
-    def __init__(self, fun,  x0, args=(), method='L-BFGS-B', jac=None,
-                 hess=None, hessp=None, bounds=None, constraints=(),
-                 tol=None, callback=None, options=None, evolution=False):
-        """ A class for handling minimization of scalar function of one or more
+    @warning_for_keywords()
+    def __init__(
+        self,
+        fun,
+        x0,
+        args=(),
+        *,
+        method="L-BFGS-B",
+        jac=None,
+        hess=None,
+        hessp=None,
+        bounds=None,
+        constraints=(),
+        tol=None,
+        callback=None,
+        options=None,
+        evolution=False,
+    ):
+        """A class for handling minimization of scalar function of one or more
         variables.
 
         Parameters
@@ -76,6 +92,7 @@ class Optimizer:
         constraints : dict or sequence of dict, optional
             Constraints definition (only for COBYLA and SLSQP).
             Each constraint is defined in a dictionary with fields:
+
                 type : str
                     Constraint type: 'eq' for equality, 'ineq' for inequality.
                 fun : callable
@@ -84,6 +101,7 @@ class Optimizer:
                     The Jacobian of `fun` (only for SLSQP).
                 args : sequence, optional
                     Extra arguments to be passed to the function and Jacobian.
+
             Equality constraint means that the constraint function result is to
             be zero whereas inequality means that it is to be non-negative.
             Note that COBYLA only supports inequality constraints.
@@ -99,10 +117,12 @@ class Optimizer:
         options : dict, optional
             A dictionary of solver options. All methods accept the following
             generic options:
+
                 maxiter : int
                     Maximum number of iterations to perform.
                 disp : bool
                     Set to True to print convergence messages.
+
             For method-specific options, see
             `show_options('minimize', method)`.
 
@@ -119,49 +139,65 @@ class Optimizer:
         self._evol_kx = None
 
         if evolution is True:
-
             self._evol_kx = []
 
             def history_of_x(kx):
                 self._evol_kx.append(kx)
-            res = minimize(fun, x0, args, method, jac, hess, hessp, bounds,
-                           constraints, tol, callback=history_of_x,
-                           options=options)
+
+            res = minimize(
+                fun,
+                x0,
+                args,
+                method,
+                jac,
+                hess,
+                hessp,
+                bounds,
+                constraints,
+                tol,
+                callback=history_of_x,
+                options=options,
+            )
 
         else:
-
-            res = minimize(fun, x0, args, method, jac, hess, hessp, bounds,
-                           constraints, tol, callback, options)
+            res = minimize(
+                fun,
+                x0,
+                args,
+                method,
+                jac,
+                hess,
+                hessp,
+                bounds,
+                constraints,
+                tol,
+                callback,
+                options,
+            )
 
         self.res = res
 
     @property
     def xopt(self):
-
-        return self.res['x']
+        return self.res["x"]
 
     @property
     def fopt(self):
-
-        return self.res['fun']
+        return self.res["fun"]
 
     @property
     def nit(self):
-
-        return self.res['nit']
+        return self.res["nit"]
 
     @property
     def nfev(self):
-
-        return self.res['nfev']
+        return self.res["nfev"]
 
     @property
     def message(self):
-
-        return self.res['message']
+        return self.res["message"]
 
     def print_summary(self):
-
         print(self.res)
 
     @property
@@ -189,23 +225,21 @@ def spdot(A, B):
     http://mail.scipy.org/pipermail/scipy-user/2010-November/027700.html
 
     """
-    if sps.issparse(A) and sps.issparse(B):
-        return A * B
-    elif sps.issparse(A) and not sps.issparse(B):
-        return (A * B).view(type=B.__class__)
-    elif not sps.issparse(A) and sps.issparse(B):
-        return (B.T * A.T).T.view(type=A.__class__)
-    else:
-        return np.dot(A, B)
+    return A @ B
 
 
-def sparse_nnls(y, X,
-                momentum=1,
-                step_size=0.01,
-                non_neg=True,
-                check_error_iter=10,
-                max_error_checks=10,
-                converge_on_sse=0.99):
+@warning_for_keywords()
+def sparse_nnls(
+    y,
+    X,
+    *,
+    momentum=1,
+    step_size=0.01,
+    non_neg=True,
+    check_error_iter=10,
+    max_error_checks=10,
+    converge_on_sse=0.99,
+):
     """
     Solve y=Xh for h, using gradient descent, with X a sparse matrix.
 
@@ -217,24 +251,24 @@ def sparse_nnls(y, X,
     X : ndarray. May be either sparse or dense. Shape (N, M)
        The regressors
 
-    momentum : float, optional (default: 1).
+    momentum : float, optional
         The persistence of the gradient.
 
-    step_size : float, optional (default: 0.01).
+    step_size : float, optional
         The increment of parameter update in each iteration
 
-    non_neg : Boolean, optional (default: True)
+    non_neg : Boolean, optional
         Whether to enforce non-negativity of the solution.
 
-    check_error_iter : int (default:10)
+    check_error_iter : int, optional
         How many rounds to run between error evaluation for
         convergence-checking.
 
-    max_error_checks : int (default: 10)
+    max_error_checks : int, optional
         Don't check errors more than this number of times if no improvement in
         r-squared is seen.
 
-    converge_on_sse : float (default: 0.99)
+    converge_on_sse : float, optional
       a percentage improvement in SSE that is required each time to say
       that things are still going well.
 
@@ -250,7 +284,7 @@ def sparse_nnls(y, X,
     h_best = h
     iteration = 1
     ss_residuals_min = np.inf  # This will keep track of the best solution
-    sse_best = np.inf   # This will keep track of the best performance so far
+    sse_best = np.inf  # This will keep track of the best performance so far
     count_bad = 0  # Number of times estimation error has gone up.
     error_checks = 0  # How many error checks have we done so far
 
@@ -260,8 +294,7 @@ def sparse_nnls(y, X,
             gradient = spdot(X.T, spdot(X, h) - y)
             gradient += momentum * gradient
             # Normalize to unit-length
-            unit_length_gradient = (gradient /
-                                    np.sqrt(np.dot(gradient, gradient)))
+            unit_length_gradient = gradient / np.sqrt(np.dot(gradient, gradient))
             # Update the parameters in the direction of the gradient:
             h -= step_size * unit_length_gradient
             if non_neg:
@@ -304,13 +337,14 @@ class SKLearnLinearSolver(metaclass=abc.ABCMeta):
     such that an estimate of y can be calculated as:
     `y_hat = np.dot(X, SKLearnLinearSolver.coef_.T)`
     """
+
     def __init__(self, *args, **kwargs):
         self._args = args
         self._kwargs = kwargs
 
     @abc.abstractmethod
     def fit(self, X, y):
-        """Implement for all derived classes """
+        """Implement for all derived classes"""
 
     def predict(self, X):
         """
@@ -335,12 +369,17 @@ class NonNegativeLeastSquares(SKLearnLinearSolver):
     A sklearn-like interface to scipy.optimize.nnls
 
     """
+
     def fit(self, X, y):
         """
         Fit the NonNegativeLeastSquares linear model to data
 
         Parameters
         ----------
+        X : array-like (n_samples, n_features)
+            Samples.
+        y : array-like (n_samples,)
+            Target values.
 
         """
         coef, rnorm = opt.nnls(X, y)
@@ -349,9 +388,11 @@ class NonNegativeLeastSquares(SKLearnLinearSolver):
 
 
 class PositiveDefiniteLeastSquares:
+    @warning_for_keywords()
+    def __init__(self, m, *, A=None, L=None):
+        r"""Regularized least squares with linear matrix inequality constraints.
 
-    def __init__(self, m, A=None, L=None):
-        r""" Regularized least squares with linear matrix inequality constraints
+        See :footcite:p:`DelaHaije2020` for further details about the method.
 
         Generate a CVXPY representation of a regularized least squares
         optimization problem subject to linear matrix inequality constraints.
@@ -360,9 +401,9 @@ class PositiveDefiniteLeastSquares:
         ----------
         m : int
             Positive int indicating the number of regressors.
-        A : array (t = m + k + 1, p, p) (optional)
+        A : array (t = m + k + 1, p, p), optional
             Constraint matrices $A$.
-        L : array (m, m) (optional)
+        L : array (m, m), optional
             Regularization matrix $L$.
             Default: None.
 
@@ -384,13 +425,12 @@ class PositiveDefiniteLeastSquares:
         this type.
 
         This formulation is used here mainly to enforce polynomial
-        sum-of-squares constraints on various models, as described in [1]_.
+        sum-of-squares constraints on various models, as described in
+        :footcite:p:`DelaHaije2020`.
 
         References
         ----------
-        .. [1] Dela Haije et al. "Enforcing necessary non-negativity constraints
-               for common diffusion MRI models using sum of squares
-               programming". NeuroImage 209, 2020, 116405.
+        .. footbibliography::
         """
         # Input
         self.A = A
@@ -400,21 +440,19 @@ class PositiveDefiniteLeastSquares:
         t = len(A) if A else 0
         k = t - m - 1
 
-        sparsity = [(i, j) for i in range(m) for j in range(i, m)]
-
         # Unknowns
-        self._X = cvxpy.Parameter((m, m), sparsity=sparsity)    # Design matrix
-        self._f = cvxpy.Parameter(m)    # Given solution for feasibility check
-        self._h = cvxpy.Variable(m)     # Solution to constrained problem
-        self._y = cvxpy.Parameter(m)    # Regressand
+        self._X = cvxpy.Parameter((m, m))  # Design matrix
+        self._f = cvxpy.Parameter(m)  # Given solution for feasibility check
+        self._h = cvxpy.Variable(m)  # Solution to constrained problem
+        self._y = cvxpy.Parameter(m)  # Regressand
 
         # Error output
         self._zeros = np.zeros(m)
 
         # Objective
-        c = self._X@self._h - self._y
+        c = self._X @ self._h - self._y
         if L is not None:
-            c += L@self._h
+            c += L @ self._h
 
         f_objective = cvxpy.Minimize(0)
         p_objective = cvxpy.Minimize(cvxpy.norm(c))
@@ -444,8 +482,9 @@ class PositiveDefiniteLeastSquares:
         self.unconstrained_problem = cvxpy.Problem(p_objective)
         self.feasibility_problem = cvxpy.Problem(f_objective, f_constraints)
 
-    def solve(self, design_matrix, measurements, check=False, **kwargs):
-        r""" Solve CVXPY problem
+    @warning_for_keywords()
+    def solve(self, design_matrix, measurements, *, check=False, **kwargs):
+        r"""Solve CVXPY problem
 
         Solve a CVXPY problem instance for a given design matrix and a given set
         of observations, and return the optimum.
@@ -456,12 +495,11 @@ class PositiveDefiniteLeastSquares:
             Design matrix.
         measurements : array (n)
             Measurements.
-        check : boolean (optional)
+        check : boolean, optional
             If True check whether the unconstrained optimization solution
             already satisfies the constraints, before running the constrained
             optimization. This adds overhead, but can avoid unnecessary
             constrained optimization calls.
-            Default: False
         kwargs : keyword arguments
             Arguments passed to the CVXPY solve method.
 
@@ -475,36 +513,35 @@ class PositiveDefiniteLeastSquares:
         try:
             X = np.linalg.cholesky(np.dot(design_matrix.T, design_matrix)).T
         except np.linalg.linalg.LinAlgError:
-            msg = 'Cholesky decomposition failed, returning zero array. Verify '
-            msg += 'that the data is sufficient to estimate the model '
-            msg += 'parameters, and that the design matrix has full rank.'
-            warnings.warn(msg)
+            msg = "Cholesky decomposition failed, returning zero array. Verify "
+            msg += "that the data is sufficient to estimate the model "
+            msg += "parameters, and that the design matrix has full rank."
+            warnings.warn(msg, stacklevel=2)
             return self._zeros
         self._X.value = X
-        self._y.value = np.linalg.multi_dot([X, np.linalg.pinv(design_matrix),
-                                             measurements])
+        self._y.value = np.linalg.multi_dot(
+            [X, np.linalg.pinv(design_matrix), measurements]
+        )
 
         try:
-
             # Check unconstrained solution
             if check:
-
                 # Solve unconstrained problem
                 self.unconstrained_problem.solve(**kwargs)
 
                 # Return zeros if optimization failed
                 status = self.unconstrained_problem.status
-                if status != 'optimal':
-                    msg = 'Solver failed to produce an optimum: %s.' % status
-                    warnings.warn(msg)
-                    msg = 'Optimization failed, returning zero array.'
-                    warnings.warn(msg)
+                if status != "optimal":
+                    msg = f"Solver failed to produce an optimum: {status}."
+                    warnings.warn(msg, stacklevel=2)
+                    msg = "Optimization failed, returning zero array."
+                    warnings.warn(msg, stacklevel=2)
                     return self._zeros
 
                 # Return unconstrained solution if satisfactory
                 self._f.value = self._h.value
                 self.feasibility_problem.solve(**kwargs)
-                if self.feasibility_problem.status == 'optimal':
+                if self.feasibility_problem.status == "optimal":
                     return np.asarray(self._h.value).squeeze()
 
             # Solve constrained problem
@@ -512,16 +549,15 @@ class PositiveDefiniteLeastSquares:
 
             # Show warning if solution is not optimal
             status = self.problem.status
-            if status != 'optimal':
-                msg = 'Solver failed to produce an optimum: %s.' % status
-                warnings.warn(msg)
+            if status != "optimal":
+                msg = f"Solver failed to produce an optimum: {status}."
+                warnings.warn(msg, stacklevel=2)
 
             # Return solution
             return np.asarray(self._h.value).squeeze()
 
         except cvxpy.error.SolverError:
-
             # Return zeros
-            msg = 'Optimization failed, returning zero array.'
-            warnings.warn(msg)
+            msg = "Optimization failed, returning zero array."
+            warnings.warn(msg, stacklevel=2)
             return self._zeros
