@@ -2,51 +2,49 @@
 The Sparse Fascicle Model.
 
 This is an implementation of the sparse fascicle model described in
-[Rokem2015]_. The multi b-value version of this model is described in
-[Rokem2014]_.
+:footcite:t:`Rokem2015`. The multi b-value version of this model is described
+in :footcite:t:`Rokem2014`.
 
-
-.. [Rokem2015] Ariel Rokem, Jason D. Yeatman, Franco Pestilli, Kendrick
-   N. Kay, Aviv Mezer, Stefan van der Walt, Brian A. Wandell
-   (2015). Evaluating the accuracy of diffusion MRI models in white
-   matter. PLoS ONE 10(4): e0123272. doi:10.1371/journal.pone.0123272
-
-.. [Rokem2014] Ariel Rokem, Kimberly L. Chan, Jason D. Yeatman, Franco
-   Pestilli,  Brian A. Wandell (2014). Evaluating the accuracy of diffusion
-   models at multiple b-values with cross-validation. ISMRM 2014.
+References
+----------
+.. footbibliography::
 """
+
+from collections import OrderedDict
+import gc
 import warnings
 
 import numpy as np
-import gc
-from collections import OrderedDict
 
 try:
     from numpy import nanmean
 except ImportError:
     from scipy.stats import nanmean
 
-from dipy.utils.optpkg import optional_package
-from dipy.utils.multiproc import determine_num_processes
 import dipy.core.gradients as grad
-import dipy.core.optimize as opt
-import dipy.sims.voxel as sims
-import dipy.data as dpd
-from dipy.reconst.base import ReconstModel, ReconstFit
-from dipy.reconst.cache import Cache
 from dipy.core.onetime import auto_attr
+import dipy.core.optimize as opt
+import dipy.data as dpd
+from dipy.reconst.base import ReconstFit, ReconstModel
+from dipy.reconst.cache import Cache
+import dipy.sims.voxel as sims
+from dipy.testing.decorators import warning_for_keywords
+from dipy.utils.multiproc import determine_num_processes
+from dipy.utils.optpkg import optional_package
 
-joblib, has_joblib, _ = optional_package('joblib')
-sklearn, has_sklearn, _ = optional_package('sklearn')
-lm, _, _ = optional_package('sklearn.linear_model')
+joblib, has_joblib, _ = optional_package("joblib")
+sklearn, has_sklearn, _ = optional_package("sklearn")
+lm, _, _ = optional_package("sklearn.linear_model")
 
 
 # Isotropic signal models: these are models of the part of the signal that
 # changes with b-value, but does not change with direction. This collection is
 # extensible, by inheriting from IsotropicModel/IsotropicFit below:
 
+
 # First, a helper function to derive the fit signal for these models:
-def _to_fit_iso(data, gtab, mask=None):
+@warning_for_keywords()
+def _to_fit_iso(data, gtab, *, mask=None):
     if mask is None:
         mask = np.ones(data.shape[:-1], dtype=bool)
     # Turn it into a 2D thing:
@@ -79,6 +77,7 @@ class IsotropicModel(ReconstModel):
     mean in each voxel as an estimate of the signal that does not depend on
     direction.
     """
+
     def __init__(self, gtab):
         """Initialize an IsotropicModel.
 
@@ -89,7 +88,8 @@ class IsotropicModel(ReconstModel):
         """
         ReconstModel.__init__(self, gtab)
 
-    def fit(self, data, mask=None, **kwargs):
+    @warning_for_keywords()
+    def fit(self, data, *, mask=None, **kwargs):
         """Fit an IsotropicModel.
 
         This boils down to finding the mean diffusion-weighted signal in each
@@ -120,12 +120,14 @@ class IsotropicFit(ReconstFit):
     A fit object for representing the isotropic signal as the mean of the
     diffusion-weighted signal.
     """
+
     def __init__(self, model, params):
         """Initialize an IsotropicFit object.
 
         Parameters
         ----------
         model : IsotropicModel class instance
+            Isotropic model.
         params : ndarray
             The mean isotropic model parameters (the mean diffusion-weighted
             signal in each voxel).
@@ -137,7 +139,8 @@ class IsotropicFit(ReconstFit):
         self.model = model
         self.params = params
 
-    def predict(self, gtab=None):
+    @warning_for_keywords()
+    def predict(self, *, gtab=None):
         """Predict the isotropic signal.
 
         Based on a gradient table. In this case, the (naive!) prediction will
@@ -145,7 +148,7 @@ class IsotropicFit(ReconstFit):
 
         Parameters
         ----------
-        gtab : a GradientTable class instance (optional)
+        gtab : a GradientTable class instance, optional
             Defaults to use the gtab from the IsotropicModel from which this
             fit was derived.
 
@@ -153,11 +156,11 @@ class IsotropicFit(ReconstFit):
         if gtab is None:
             gtab = self.model.gtab
         if len(self.params.shape) == 0:
-            return self.params[..., np.newaxis] + np.zeros(
-                                                        np.sum(~gtab.b0s_mask))
+            return self.params[..., np.newaxis] + np.zeros(np.sum(~gtab.b0s_mask))
         else:
             return self.params[..., np.newaxis] + np.zeros(
-                self.params.shape + (np.sum(~gtab.b0s_mask),))
+                self.params.shape + (np.sum(~gtab.b0s_mask),)
+            )
 
 
 class ExponentialIsotropicModel(IsotropicModel):
@@ -165,7 +168,9 @@ class ExponentialIsotropicModel(IsotropicModel):
     Representing the isotropic signal as a fit to an exponential decay function
     with b-values
     """
-    def fit(self, data, mask=None, **kwargs):
+
+    @warning_for_keywords()
+    def fit(self, data, *, mask=None, **kwargs):
         """
 
         Parameters
@@ -200,7 +205,9 @@ class ExponentialIsotropicFit(IsotropicFit):
     """
     A fit to the ExponentialIsotropicModel object, based on data.
     """
-    def predict(self, gtab=None):
+
+    @warning_for_keywords()
+    def predict(self, *, gtab=None):
         """
         Predict the isotropic signal, based on a gradient table. In this case,
         the prediction will be for an exponential decay with the mean
@@ -208,24 +215,29 @@ class ExponentialIsotropicFit(IsotropicFit):
 
         Parameters
         ----------
-        gtab : a GradientTable class instance (optional)
+        gtab : a GradientTable class instance, optional
             Defaults to use the gtab from the IsotropicModel from which this
             fit was derived.
         """
         if gtab is None:
             gtab = self.model.gtab
         if len(self.params.shape) == 0:
-            return np.exp(-gtab.bvals[~gtab.b0s_mask] *
-                          (np.zeros(np.sum(~gtab.b0s_mask)) +
-                          self.params[..., np.newaxis]))
+            return np.exp(
+                -gtab.bvals[~gtab.b0s_mask]
+                * (np.zeros(np.sum(~gtab.b0s_mask)) + self.params[..., np.newaxis])
+            )
         else:
-            return np.exp(-gtab.bvals[~gtab.b0s_mask] *
-                          (np.zeros((self.params.shape[0],
-                                     np.sum(~gtab.b0s_mask))) +
-                          self.params[..., np.newaxis]))
+            return np.exp(
+                -gtab.bvals[~gtab.b0s_mask]
+                * (
+                    np.zeros((self.params.shape[0], np.sum(~gtab.b0s_mask)))
+                    + self.params[..., np.newaxis]
+                )
+            )
 
 
-def sfm_design_matrix(gtab, sphere, response, mode='signal'):
+@warning_for_keywords()
+def sfm_design_matrix(gtab, sphere, response, *, mode="signal"):
     """
     Construct the SFM design matrix
 
@@ -271,75 +283,73 @@ def sfm_design_matrix(gtab, sphere, response, mode='signal'):
     >>> sphere = dpd.get_sphere()
     >>> from dipy.reconst.sfm import sfm_design_matrix
 
-    A canonical tensor approximating corpus-callosum voxels [Rokem2014]_:
+    A canonical tensor approximating corpus-callosum voxels
+    :footcite:p`Rokem2014`:
 
     >>> tensor_matrix = sfm_design_matrix(gtab, sphere,
     ...                                   [0.0015, 0.0005, 0.0005])
 
-    A 'stick' function ([Behrens2007]_):
+    A 'stick' function :footcite:p`Behrens2007`:
 
     >>> stick_matrix = sfm_design_matrix(gtab, sphere, [0.001, 0, 0])
 
-    Notes
-    -----
-    .. [Rokem2015] Ariel Rokem, Jason D. Yeatman, Franco Pestilli, Kendrick
-       N. Kay, Aviv Mezer, Stefan van der Walt, Brian A. Wandell
-       (2015). Evaluating the accuracy of diffusion MRI models in white
-       matter. PLoS ONE 10(4): e0123272. doi:10.1371/journal.pone.0123272
-
-    .. [Rokem2014] Ariel Rokem, Kimberly L. Chan, Jason D. Yeatman, Franco
-       Pestilli,  Brian A. Wandell (2014). Evaluating the accuracy of diffusion
-       models at multiple b-values with cross-validation. ISMRM 2014.
-
-    .. [Behrens2007] Behrens TEJ, Berg HJ, Jbabdi S, Rushworth MFS, Woolrich MW
-       (2007): Probabilistic diffusion tractography with multiple fibre
-       orientations: What can we gain? Neuroimage 34:144-55.
+    References
+    ----------
+    .. footbibliography::
     """
-    if mode == 'signal':
+    if mode == "signal":
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
-            mat_gtab = grad.gradient_table(gtab.bvals[~gtab.b0s_mask],
-                                           gtab.bvecs[~gtab.b0s_mask])
+            mat_gtab = grad.gradient_table(
+                gtab.bvals[~gtab.b0s_mask], bvecs=gtab.bvecs[~gtab.b0s_mask]
+            )
         # Preallocate:
-        mat = np.empty((np.sum(~gtab.b0s_mask),
-                        sphere.vertices.shape[0]))
-    elif mode == 'odf':
+        mat = np.empty((np.sum(~gtab.b0s_mask), sphere.vertices.shape[0]))
+    elif mode == "odf":
         mat = np.empty((gtab.x.shape[0], sphere.vertices.shape[0]))
 
     # Calculate column-wise:
     for ii, this_dir in enumerate(sphere.vertices):
         # Rotate the canonical tensor towards this vertex and calculate the
         # signal you would have gotten in the direction
-        if mode == 'signal':
+        if mode == "signal":
             # For regressors based on the single tensor, remove $e^{-bD}$
             mat[:, ii] = sims.single_tensor(
-                mat_gtab,
-                evals=response,
-                evecs=sims.all_tensor_evecs(this_dir)
+                mat_gtab, evals=response, evecs=sims.all_tensor_evecs(this_dir)
             ) - np.exp(-mat_gtab.bvals * np.mean(response))
 
-        elif mode == 'odf':
+        elif mode == "odf":
             # Stick function
             if response[1] == 0 or response[2] == 0:
-                mat[sphere.find_closest(sims.all_tensor_evecs(this_dir)[0]),
-                    ii] = 1
+                mat[sphere.find_closest(sims.all_tensor_evecs(this_dir)[0]), ii] = 1
             else:
                 mat[:, ii] = sims.single_tensor_odf(
-                    gtab.vertices, evals=response,
-                    evecs=sims.all_tensor_evecs(this_dir))
+                    gtab.vertices, evals=response, evecs=sims.all_tensor_evecs(this_dir)
+                )
     return mat
 
 
 class SparseFascicleModel(ReconstModel, Cache):
-    def __init__(self, gtab, sphere=None, response=(0.0015, 0.0005, 0.0005),
-                 solver='ElasticNet', l1_ratio=0.5, alpha=0.001,
-                 isotropic=None, seed=42):
+    @warning_for_keywords()
+    def __init__(
+        self,
+        gtab,
+        *,
+        sphere=None,
+        response=(0.0015, 0.0005, 0.0005),
+        solver="ElasticNet",
+        l1_ratio=0.5,
+        alpha=0.001,
+        isotropic=None,
+        seed=42,
+    ):
         """
         Initialize a Sparse Fascicle Model
 
         Parameters
         ----------
         gtab : GradientTable class instance
+            Gradient table.
 
         sphere : Sphere class instance, optional
             A sphere on which coefficients will be estimated. Default:
@@ -348,7 +358,6 @@ class SparseFascicleModel(ReconstModel, Cache):
         response : (3,) array-like, optional
             The eigenvalues of a canonical tensor to be used as the response
             function of single-fascicle signals.
-            Default:[0.0015, 0.0005, 0.0005]
 
         solver : string, or initialized linear model object.
             This will determine the algorithm used to solve the set of linear
@@ -359,15 +368,14 @@ class SparseFascicleModel(ReconstModel, Cache):
             `sklearn.linear_model.ElasticNet`, `sklearn.linear_model.Lasso` or
             `sklearn.linear_model.Ridge` and other objects that inherit from
             `sklearn.base.RegressorMixin`.
-            Default: 'ElasticNet'.
 
         l1_ratio : float, optional
             Sets the balance between L1 and L2 regularization in ElasticNet
-            [Zou2005]_. Default: 0.5
+            :footcite:p`Zou2005`.
 
         alpha : float, optional
             Sets the balance between least-squares error and L1/L2
-            regularization in ElasticNet [Zou2005]_. Default: 0.001
+            regularization in ElasticNet :footcite:p`Zou2005`.
 
         isotropic : IsotropicModel class instance
             This is a class that implements the function that calculates the
@@ -380,15 +388,12 @@ class SparseFascicleModel(ReconstModel, Cache):
 
         Notes
         -----
-        This is an implementation of the SFM, described in [Rokem2015]_.
+        This is an implementation of the SFM, described in
+        :footcite:p`Rokem2015`.
 
-        .. [Rokem2014] Ariel Rokem, Jason D. Yeatman, Franco Pestilli, Kendrick
-           N. Kay, Aviv Mezer, Stefan van der Walt, Brian A. Wandell
-           (2014). Evaluating the accuracy of diffusion MRI models in white
-           matter. PLoS ONE 10(4): e0123272. doi:10.1371/journal.pone.0123272
-
-        .. [Zou2005] Zou H, Hastie T (2005). Regularization and variable
-           selection via the elastic net. J R Stat Soc B:301-320
+        References
+        ----------
+        .. footbibliography::
         """
         ReconstModel.__init__(self, gtab)
 
@@ -400,15 +405,22 @@ class SparseFascicleModel(ReconstModel, Cache):
             isotropic = IsotropicModel
 
         self.isotropic = isotropic
-        if solver == 'ElasticNet':
-            self.solver = lm.ElasticNet(l1_ratio=l1_ratio, alpha=alpha,
-                                        positive=True, warm_start=False,
-                                        random_state=seed)
-        elif solver in ('NNLS', 'nnls'):
+        if solver == "ElasticNet":
+            self.solver = lm.ElasticNet(
+                l1_ratio=l1_ratio,
+                alpha=alpha,
+                positive=True,
+                warm_start=False,
+                random_state=seed,
+            )
+        elif solver in ("NNLS", "nnls"):
             self.solver = opt.NonNegativeLeastSquares()
 
-        elif (isinstance(solver, opt.SKLearnLinearSolver) or
-              has_sklearn and isinstance(solver, sklearn.base.RegressorMixin)):
+        elif (
+            isinstance(solver, opt.SKLearnLinearSolver)
+            or has_sklearn
+            and isinstance(solver, sklearn.base.RegressorMixin)
+        ):
             self.solver = solver
 
         else:
@@ -417,7 +429,7 @@ class SparseFascicleModel(ReconstModel, Cache):
             if not has_sklearn:
                 w = sklearn._msg + "\nAlternatively, you can use 'nnls' method "
                 w += "to fit the SparseFascicleModel"
-                warnings.warn(w)
+                warnings.warn(w, stacklevel=2)
             e_s = "The `solver` key-word argument needs to be: "
             e_s += "'ElasticNet', 'NNLS', or a "
             e_s += "`dipy.optimize.SKLearnLinearSolver` object"
@@ -434,10 +446,10 @@ class SparseFascicleModel(ReconstModel, Cache):
             The design matrix, where each column is a rotated version of the
             response function.
         """
-        return sfm_design_matrix(self.gtab, self.sphere, self.response,
-                                 'signal')
+        return sfm_design_matrix(self.gtab, self.sphere, self.response, mode="signal")
 
-    def _fit_solver2voxels(self, isopredict, vox_data, vox, parallel=False):
+    @warning_for_keywords()
+    def _fit_solver2voxels(self, isopredict, vox_data, vox, *, parallel=False):
         # In voxels in which S0 is 0, we just want to keep the
         # parameters at all-zeros, and avoid nasty sklearn errors:
         if not (np.any(~np.isfinite(vox_data)) or np.all(vox_data == 0)):
@@ -445,9 +457,11 @@ class SparseFascicleModel(ReconstModel, Cache):
                 warnings.simplefilter("ignore")
 
                 if parallel:
-                    coef = {vox: self.solver.fit(
-                        self.design_matrix, vox_data - isopredict[vox]
-                    ).coef_}
+                    coef = {
+                        vox: self.solver.fit(
+                            self.design_matrix, vox_data - isopredict[vox]
+                        ).coef_
+                    }
                 else:
                     coef = self.solver.fit(
                         self.design_matrix, vox_data - isopredict[vox]
@@ -459,8 +473,10 @@ class SparseFascicleModel(ReconstModel, Cache):
                 return np.zeros(self.design_matrix.shape[-1])
         return coef
 
-    def fit(self, data, mask=None, num_processes=1,
-            parallel_backend='multiprocessing'):
+    @warning_for_keywords()
+    def fit(
+        self, data, *, mask=None, num_processes=1, parallel_backend="multiprocessing"
+    ):
         """
         Fit the SparseFascicleModel object to data.
 
@@ -485,6 +501,7 @@ class SparseFascicleModel(ReconstModel, Cache):
         parallel_backend: str, ParallelBackendBase instance or None
             Specify the parallelization backend implementation.
             Supported backends are:
+
             - "loky" used by default, can induce some
               communication and memory overhead when exchanging input and
               output data with the worker Python processes.
@@ -497,7 +514,6 @@ class SparseFascicleModel(ReconstModel, Cache):
               explicitly releases the GIL (for instance a Cython loop wrapped
               in a "with nogil" block or an expensive call to a library such
               as NumPy).
-            Default: 'multiprocessing'.
 
         Returns
         -------
@@ -511,7 +527,7 @@ class SparseFascicleModel(ReconstModel, Cache):
             # Check for valid shape of the mask
             if mask.shape != data.shape[:-1]:
                 raise ValueError("Mask is not the same shape as data.")
-            mask = np.array(mask, dtype=bool, copy=False)
+            mask = np.asarray(mask, dtype=bool)
             data_in_mask = np.reshape(data[mask], (-1, data.shape[-1]))
 
         # Fitting is done on the relative signal (S/S0):
@@ -519,11 +535,9 @@ class SparseFascicleModel(ReconstModel, Cache):
         if not flat_S0.size or not flat_S0.max():
             flat_S = np.zeros(data_in_mask[..., ~self.gtab.b0s_mask].shape)
         else:
-            flat_S = (data_in_mask[..., ~self.gtab.b0s_mask] /
-                      flat_S0[..., None])
-        isotropic = self.isotropic(self.gtab).fit(data, mask)
-        flat_params = np.zeros((data_in_mask.shape[0],
-                                self.design_matrix.shape[-1]))
+            flat_S = data_in_mask[..., ~self.gtab.b0s_mask] / flat_S0[..., None]
+        isotropic = self.isotropic(self.gtab).fit(data, mask=mask)
+        flat_params = np.zeros((data_in_mask.shape[0], self.design_matrix.shape[-1]))
         del data_in_mask
         gc.collect()
 
@@ -537,14 +551,15 @@ class SparseFascicleModel(ReconstModel, Cache):
             num_processes = determine_num_processes(num_processes)
 
         if num_processes > 1 and has_joblib:
-            with joblib.Parallel(n_jobs=num_processes,
-                                 backend=parallel_backend,
-                                 mmap_mode='r+') as parallel:
+            with joblib.Parallel(
+                n_jobs=num_processes, backend=parallel_backend, mmap_mode="r+"
+            ) as parallel:
                 out = parallel(
-                    joblib.delayed(self._fit_solver2voxels)(isopredict,
-                                                            vox_data, vox,
-                                                            True) for
-                    vox, vox_data in enumerate(flat_S))
+                    joblib.delayed(self._fit_solver2voxels)(
+                        isopredict, vox_data, vox, True
+                    )
+                    for vox, vox_data in enumerate(flat_S)
+                )
 
             del parallel
 
@@ -552,26 +567,30 @@ class SparseFascicleModel(ReconstModel, Cache):
             for d in out:
                 flat_params_dict.update(d)
             flat_params = np.concatenate(
-                [np.array(i).reshape(1, flat_params.shape[1])
-                 for i in list(OrderedDict(
-                    sorted(flat_params_dict.items(),
-                           key=lambda x: int(x[0]))).values())])
+                [
+                    np.array(i).reshape(1, flat_params.shape[1])
+                    for i in list(
+                        OrderedDict(
+                            sorted(flat_params_dict.items(), key=lambda x: int(x[0]))
+                        ).values()
+                    )
+                ]
+            )
         else:
             for vox, vox_data in enumerate(flat_S):
-                flat_params[vox] = self._fit_solver2voxels(isopredict,
-                                                           vox_data, vox,
-                                                           False)
+                flat_params[vox] = self._fit_solver2voxels(
+                    isopredict, vox_data, vox, parallel=False
+                )
 
         del isopredict, flat_S
         gc.collect()
 
         if mask is None:
-            out_shape = data.shape[:-1] + (-1, )
+            out_shape = data.shape[:-1] + (-1,)
             beta = flat_params.reshape(out_shape)
             S0 = flat_S0.reshape(data.shape[:-1])
         else:
-            beta = np.zeros(data.shape[:-1] +
-                            (self.design_matrix.shape[-1],))
+            beta = np.zeros(data.shape[:-1] + (self.design_matrix.shape[-1],))
             beta[mask, :] = flat_params
             S0 = np.zeros(data.shape[:-1])
             S0[mask] = flat_S0
@@ -580,7 +599,6 @@ class SparseFascicleModel(ReconstModel, Cache):
 
 
 class SparseFascicleFit(ReconstFit):
-
     def __init__(self, model, beta, S0, iso):
         """
         Initialize a SparseFascicleFit class instance
@@ -621,18 +639,19 @@ class SparseFascicleFit(ReconstFit):
         odf :  ndarray of shape (x, y, z, sphere.vertices.shape[0])
 
         """
-        odf_matrix = self.model.cache_get('odf_matrix', key=sphere)
+        odf_matrix = self.model.cache_get("odf_matrix", key=sphere)
         if odf_matrix is None:
-            odf_matrix = sfm_design_matrix(sphere, self.model.sphere,
-                                           self.model.response, mode='odf')
-            self.model.cache_set('odf_matrix', key=sphere, value=odf_matrix)
+            odf_matrix = sfm_design_matrix(
+                sphere, self.model.sphere, self.model.response, mode="odf"
+            )
+            self.model.cache_set("odf_matrix", key=sphere, value=odf_matrix)
 
-        return np.dot(odf_matrix,
-                      self.beta.reshape(-1,
-                                        self.beta.shape[-1]).T).T.reshape(
-            self.beta.shape[:-1] + (odf_matrix.shape[0], ))
+        return np.dot(
+            odf_matrix, self.beta.reshape(-1, self.beta.shape[-1]).T
+        ).T.reshape(self.beta.shape[:-1] + (odf_matrix.shape[0],))
 
-    def predict(self, gtab=None, response=None, S0=None):
+    @warning_for_keywords()
+    def predict(self, *, gtab=None, response=None, S0=None):
         """
         Predict the signal based on the SFM parameters
 
@@ -665,19 +684,18 @@ class SparseFascicleFit(ReconstFit):
         else:
             _matrix = sfm_design_matrix(gtab, self.model.sphere, response)
         # Get them all at once:
-        pred_weighted = np.dot(_matrix,
-                               self.beta.reshape(
-                                   -1, self.beta.shape[-1]).T).T.reshape(
-            self.beta.shape[:-1] + (_matrix.shape[0],))
+        pred_weighted = np.dot(
+            _matrix, self.beta.reshape(-1, self.beta.shape[-1]).T
+        ).T.reshape(self.beta.shape[:-1] + (_matrix.shape[0],))
 
         if S0 is None:
             S0 = self.S0
         if isinstance(S0, np.ndarray):
             S0 = S0[..., None]
 
-        pre_pred_sig = S0 * (pred_weighted +
-                             self.iso.predict(gtab).reshape(
-                                 pred_weighted.shape))
+        pre_pred_sig = S0 * (
+            pred_weighted + self.iso.predict(gtab=gtab).reshape(pred_weighted.shape)
+        )
         pred_sig = np.zeros(pre_pred_sig.shape[:-1] + (gtab.bvals.shape[0],))
         pred_sig[..., ~gtab.b0s_mask] = pre_pred_sig
         pred_sig[..., gtab.b0s_mask] = S0
