@@ -1,27 +1,42 @@
 from warnings import warn
-from packaging.version import Version
 
 import numpy as np
+from packaging.version import Version
 
 from dipy import __version__ as horizon_version
 from dipy.io.stateful_tractogram import Space, StatefulTractogram
 from dipy.io.streamline import save_tractogram
+from dipy.testing.decorators import warning_for_keywords
 from dipy.tracking.streamline import Streamlines
 from dipy.utils.optpkg import optional_package
 from dipy.viz.gmem import GlobalHorizon
-from dipy.viz.horizon.tab import (ClustersTab, PeaksTab, ROIsTab, SlicesTab,
-                                  TabManager, build_label, SurfaceTab)
-from dipy.viz.horizon.visualizer import (ClustersVisualizer, SlicesVisualizer,
-                                         SurfaceVisualizer, PeaksVisualizer)
-from dipy.viz.horizon.util import (check_img_dtype, check_img_shapes,
-                                   unpack_image, is_binary_image,
-                                   unpack_surface, check_peak_size)
+from dipy.viz.horizon.tab import (
+    ClustersTab,
+    PeaksTab,
+    ROIsTab,
+    SlicesTab,
+    SurfaceTab,
+    TabManager,
+    build_label,
+)
+from dipy.viz.horizon.util import (
+    check_img_dtype,
+    check_img_shapes,
+    check_peak_size,
+    unpack_image,
+    unpack_surface,
+)
+from dipy.viz.horizon.visualizer import (
+    ClustersVisualizer,
+    PeaksVisualizer,
+    SlicesVisualizer,
+    SurfaceVisualizer,
+)
 
-fury, has_fury, setup_module = optional_package('fury', min_version="0.10.0")
+fury, has_fury, setup_module = optional_package("fury", min_version="0.10.0")
 
 if has_fury:
-    from fury import __version__ as fury_version
-    from fury import actor, ui, window
+    from fury import __version__ as fury_version, actor, ui, window
     from fury.colormap import distinguishable_colormap
 
 
@@ -40,18 +55,37 @@ HELP_MESSAGE = """
 
 
 class Horizon:
-
-    def __init__(self, tractograms=None, images=None, pams=None, surfaces=None,
-                 cluster=False, rgb=False, cluster_thr=15.0,
-                 random_colors=None, length_gt=0, length_lt=1000,
-                 clusters_gt=0, clusters_lt=10000,
-                 world_coords=True, interactive=True, out_png='tmp.png',
-                 recorded_events=None, return_showm=False, bg_color=(0, 0, 0),
-                 order_transparent=True, buan=False, buan_colors=None,
-                 roi_images=False, roi_colors=(1, 0, 0),
-                 surface_colors=[(1, 0, 0)]):
+    @warning_for_keywords()
+    def __init__(
+        self,
+        *,
+        tractograms=None,
+        images=None,
+        pams=None,
+        surfaces=None,
+        cluster=False,
+        rgb=False,
+        cluster_thr=15.0,
+        random_colors=None,
+        length_gt=0,
+        length_lt=1000,
+        clusters_gt=0,
+        clusters_lt=10000,
+        world_coords=True,
+        interactive=True,
+        out_png="tmp.png",
+        recorded_events=None,
+        return_showm=False,
+        bg_color=(0, 0, 0),
+        order_transparent=True,
+        buan=False,
+        buan_colors=None,
+        roi_images=False,
+        roi_colors=(1, 0, 0),
+        surface_colors=((1, 0, 0),),
+    ):
         """Interactive medical visualization - Invert the Horizon!
-
+        :footcite:p:`Garyfallidis2019`.
 
         Parameters
         ----------
@@ -103,7 +137,7 @@ class Horizon:
             File path to replay recorded events
         return_showm : bool
             Return ShowManager object. Used only at Python level. Can be used
-            for extending Horizon's cababilities externally and for testing
+            for extending Horizon's capabilities externally and for testing
             purposes.
         bg_color : ndarray or list or tuple
             Define the background color of the scene.
@@ -123,18 +157,17 @@ class Horizon:
 
         References
         ----------
-        .. [Horizon_ISMRM19] Garyfallidis E., M-A. Cote, B.Q. Chandio,
-            S. Fadnavis, J. Guaje, R. Aggarwal, E. St-Onge, K.S. Juneja,
-            S. Koudoro, D. Reagan, DIPY Horizon: fast, modular, unified and
-            adaptive visualization, Proceedings of: International Society of
-            Magnetic Resonance in Medicine (ISMRM), Montreal, Canada, 2019.
+        .. footbibliography::
         """
         if not has_fury:
-            raise ImportError('Horizon requires FURY. Please install it '
-                              'with pip install fury')
-        if Version(fury_version) < Version('0.10.0'):
-            ValueError('Horizon requires FURY version 0.10.0 or higher.'
-                       ' Please upgrade FURY with pip install -U fury.')
+            raise ImportError(
+                "Horizon requires FURY. Please install it with pip install fury"
+            )
+        if Version(fury_version) < Version("0.10.0"):
+            ValueError(
+                "Horizon requires FURY version 0.10.0 or higher."
+                " Please upgrade FURY with pip install -U fury."
+            )
 
         self.cluster = cluster
         self.rgb = rgb
@@ -157,6 +190,7 @@ class Horizon:
         self.cla = {}  # holds cluster actors
         self.recorded_events = recorded_events
         self.show_m = None
+        self._scene = None
         self.return_showm = return_showm
         self.bg_color = bg_color
         self.order_transparent = order_transparent
@@ -170,7 +204,7 @@ class Horizon:
 
         if self.random_colors is not None:
             if not self.random_colors:
-                self.random_colors = ['tracts', 'rois']
+                self.random_colors = ["tracts", "rois"]
         else:
             self.random_colors = []
 
@@ -194,14 +228,14 @@ class Horizon:
         min_length = np.min(lengths)
         min_size = np.min(sizes)
         for cent in centroid_actors:
-            if centroid_actors[cent]['selected']:
-                if not centroid_actors[cent]['expanded']:
-                    len_ = centroid_actors[cent]['length']
-                    sz_ = centroid_actors[cent]['size']
-                    if (len_ >= min_length and sz_ >= min_size):
-                        centroid_actors[cent]['actor'].VisibilityOn()
+            if centroid_actors[cent]["selected"]:
+                if not centroid_actors[cent]["expanded"]:
+                    len_ = centroid_actors[cent]["length"]
+                    sz_ = centroid_actors[cent]["size"]
+                    if len_ >= min_length and sz_ >= min_size:
+                        centroid_actors[cent]["actor"].VisibilityOn()
                         cent.VisibilityOff()
-                        centroid_actors[cent]['expanded'] = 1
+                        centroid_actors[cent]["expanded"] = 1
         self.show_m.render()
 
     # TODO: Move to another class/module
@@ -212,15 +246,15 @@ class Horizon:
         min_length = np.min(lengths)
         min_size = np.min(sizes)
         for cent in centroid_actors:
-            valid_length = centroid_actors[cent]['length'] >= min_length
-            valid_size = centroid_actors[cent]['size'] >= min_size
+            valid_length = centroid_actors[cent]["length"] >= min_length
+            valid_size = centroid_actors[cent]["size"] >= min_size
             if self.__hide_centroids:
                 if valid_length or valid_size:
-                    if centroid_actors[cent]['selected'] == 0:
+                    if centroid_actors[cent]["selected"] == 0:
                         cent.VisibilityOff()
             else:
                 if valid_length and valid_size:
-                    if centroid_actors[cent]['selected'] == 0:
+                    if centroid_actors[cent]["selected"] == 0:
                         cent.VisibilityOn()
         self.__hide_centroids = not self.__hide_centroids
         self.show_m.render()
@@ -234,14 +268,14 @@ class Horizon:
         min_length = np.min(lengths)
         min_size = np.min(sizes)
         for cent in centroid_actors:
-            valid_length = centroid_actors[cent]['length'] >= min_length
-            valid_size = centroid_actors[cent]['size'] >= min_size
+            valid_length = centroid_actors[cent]["length"] >= min_length
+            valid_size = centroid_actors[cent]["size"] >= min_size
             if valid_length and valid_size:
-                centroid_actors[cent]['selected'] = (
-                    not centroid_actors[cent]['selected'])
-                clus = centroid_actors[cent]['actor']
-                cluster_actors[clus]['selected'] = (
-                    centroid_actors[cent]['selected'])
+                centroid_actors[cent]["selected"] = not centroid_actors[cent][
+                    "selected"
+                ]
+                clus = centroid_actors[cent]["actor"]
+                cluster_actors[clus]["selected"] = centroid_actors[cent]["selected"]
         self.show_m.render()
 
     def __key_press_events(self, obj, event):
@@ -249,7 +283,7 @@ class Horizon:
         # TODO: Move to another class/module
         if self.cluster:
             # retract help panel
-            if key in ('o', 'O'):
+            if key in ("o", "O"):
                 panel_size = self.help_panel._get_size()
                 if self.__help_visible:
                     new_pos = np.array(self.__win_size) - 10
@@ -259,22 +293,22 @@ class Horizon:
                     self.__help_visible = True
                 self.help_panel._set_position(new_pos)
                 self.show_m.render()
-            if key in ('a', 'A'):
+            if key in ("a", "A"):
                 self.__show_all()
-            if key in ('e', 'E'):
+            if key in ("e", "E"):
                 self.__expand()
             # hide on/off unselected centroids
-            if key in ('h', 'H'):
+            if key in ("h", "H"):
                 self.__hide()
             # invert selection
-            if key in ('i', 'I'):
+            if key in ("i", "I"):
                 self.__invert()
-            if key in ('r', 'R'):
+            if key in ("r", "R"):
                 self.__reset()
             # save current result
-            if key in ('s', 'S'):
+            if key in ("s", "S"):
                 self.__save()
-            if key in ('y', 'Y'):
+            if key in ("y", "Y"):
                 self.__new_window()
 
     # TODO: Move to another class/module
@@ -284,19 +318,28 @@ class Horizon:
         active_streamlines = Streamlines()
         for bundle in cluster_actors.keys():
             if bundle.GetVisibility():
-                t = cluster_actors[bundle]['tractogram']
-                c = cluster_actors[bundle]['cluster']
+                t = cluster_actors[bundle]["tractogram"]
+                c = cluster_actors[bundle]["cluster"]
                 indices = tractogram_clusters[t][c]
                 active_streamlines.extend(Streamlines(indices))
 
         # Using the header of the first of the tractograms
         active_sft = StatefulTractogram(
-            active_streamlines, self.tractograms[0], Space.RASMM)
+            active_streamlines, self.tractograms[0], Space.RASMM
+        )
         hz2 = Horizon(
-            [active_sft], self.images, cluster=True,
-            cluster_thr=self.cluster_thr/2., random_colors=self.random_colors,
-            length_lt=np.inf, length_gt=0, clusters_lt=np.inf,
-            clusters_gt=0, world_coords=True, interactive=True)
+            tractograms=[active_sft],
+            images=self.images,
+            cluster=True,
+            cluster_thr=self.cluster_thr / 2.0,
+            random_colors=self.random_colors,
+            length_lt=np.inf,
+            length_gt=0,
+            clusters_lt=np.inf,
+            clusters_gt=0,
+            world_coords=True,
+            interactive=True,
+        )
         ren2 = hz2.build_scene()
         hz2.build_show(ren2)
 
@@ -308,12 +351,12 @@ class Horizon:
         min_length = np.min(lengths)
         min_size = np.min(sizes)
         for cent in centroid_actors:
-            valid_length = centroid_actors[cent]['length'] >= min_length
-            valid_size = centroid_actors[cent]['size'] >= min_size
+            valid_length = centroid_actors[cent]["length"] >= min_length
+            valid_size = centroid_actors[cent]["size"] >= min_size
             if valid_length and valid_size:
-                centroid_actors[cent]['actor'].VisibilityOff()
+                centroid_actors[cent]["actor"].VisibilityOff()
                 cent.VisibilityOn()
-                centroid_actors[cent]['expanded'] = 0
+                centroid_actors[cent]["expanded"] = 0
         self.show_m.render()
 
     # TODO: Move to another class/module
@@ -323,17 +366,18 @@ class Horizon:
         saving_streamlines = Streamlines()
         for bundle in cluster_actors.keys():
             if bundle.GetVisibility():
-                t = cluster_actors[bundle]['tractogram']
-                c = cluster_actors[bundle]['cluster']
+                t = cluster_actors[bundle]["tractogram"]
+                c = cluster_actors[bundle]["cluster"]
                 indices = tractogram_clusters[t][c]
                 saving_streamlines.extend(Streamlines(indices))
-        print('Saving result in tmp.trk')
+        print("Saving result in tmp.trk")
 
         # Using the header of the first of the tractograms
         sft_new = StatefulTractogram(
-            saving_streamlines, self.tractograms[0], Space.RASMM)
-        save_tractogram(sft_new, 'tmp.trk', bbox_valid_check=False)
-        print('Saved!')
+            saving_streamlines, self.tractograms[0], Space.RASMM
+        )
+        save_tractogram(sft_new, "tmp.trk", bbox_valid_check=False)
+        print("Saved!")
 
     # TODO: Move to another class/module
     def __show_all(self):
@@ -345,23 +389,21 @@ class Horizon:
         min_size = np.min(sizes)
         if self.__select_all:
             for cent in centroid_actors:
-                valid_length = centroid_actors[cent]['length'] >= min_length
-                valid_size = centroid_actors[cent]['size'] >= min_size
+                valid_length = centroid_actors[cent]["length"] >= min_length
+                valid_size = centroid_actors[cent]["size"] >= min_size
                 if valid_length and valid_size:
-                    centroid_actors[cent]['selected'] = 0
-                    clus = centroid_actors[cent]['actor']
-                    cluster_actors[clus]['selected'] = (
-                        centroid_actors[cent]['selected'])
+                    centroid_actors[cent]["selected"] = 0
+                    clus = centroid_actors[cent]["actor"]
+                    cluster_actors[clus]["selected"] = centroid_actors[cent]["selected"]
             self.__select_all = False
         else:
             for cent in centroid_actors:
-                valid_length = centroid_actors[cent]['length'] >= min_length
-                valid_size = centroid_actors[cent]['size'] >= min_size
+                valid_length = centroid_actors[cent]["length"] >= min_length
+                valid_size = centroid_actors[cent]["size"] >= min_size
                 if valid_length and valid_size:
-                    centroid_actors[cent]['selected'] = 1
-                    clus = centroid_actors[cent]['actor']
-                    cluster_actors[clus]['selected'] = (
-                        centroid_actors[cent]['selected'])
+                    centroid_actors[cent]["selected"] = 1
+                    clus = centroid_actors[cent]["actor"]
+                    cluster_actors[clus]["selected"] = centroid_actors[cent]["selected"]
             self.__select_all = True
         self.show_m.render()
 
@@ -390,25 +432,42 @@ class Horizon:
         """
         self.show_m.render()
 
-    def build_show(self, scene):
+    def _update_actors(self, actors):
+        """Update actors in the scene. It essentially brings them forward in
+        the stack.
 
-        title = 'Horizon ' + horizon_version
+        Parameters
+        ----------
+        actors : list
+            list of FURY actors.
+        """
+        self._scene.rm(*actors)
+        self._scene.add(*actors)
+
+    def build_show(self, scene):
+        self._scene = scene
+
+        title = f"Horizon {horizon_version}"
         self.show_m = window.ShowManager(
-            scene, title=title, size=(1920, 1080), reset_camera=False,
-            order_transparent=self.order_transparent)
+            scene=scene,
+            title=title,
+            size=(1920, 1080),
+            reset_camera=False,
+            order_transparent=self.order_transparent,
+        )
 
         if len(self.tractograms) > 0:
-
             if self.cluster:
                 self.__clusters_visualizer = ClustersVisualizer(
-                    self.show_m, scene, self.tractograms)
+                    self.show_m, scene, self.tractograms
+                )
 
             color_ind = 0
 
             for t, sft in enumerate(self.tractograms):
                 streamlines = sft.streamlines
 
-                if 'tracts' in self.random_colors:
+                if "tracts" in self.random_colors:
                     colors = next(self.color_gen)
                 else:
                     colors = None
@@ -416,12 +475,14 @@ class Horizon:
                 if not self.world_coords:
                     # TODO: Get affine from a StatefullTractogram
                     raise ValueError(
-                        'Currently native coordinates are not supported for '
-                        'streamlines.')
+                        "Currently native coordinates are not supported for "
+                        "streamlines."
+                    )
 
                 if self.cluster:
                     self.__clusters_visualizer.add_cluster_actors(
-                        t, streamlines, self.cluster_thr, colors)
+                        t, streamlines, self.cluster_thr, colors
+                    )
                 else:
                     if self.buan:
                         colors = self.buan_colors[color_ind]
@@ -439,17 +500,21 @@ class Horizon:
                 # Information panel
                 # It will be changed once all the elements wrapped in horizon
                 # elements.
-                text_block = build_label(HELP_MESSAGE, 18)
-                text_block.message = HELP_MESSAGE
+                text_block = build_label(HELP_MESSAGE, font_size=18)
 
                 self.help_panel = ui.Panel2D(
-                    size=(300, 200), position=(1615, 875), color=(.8, .8, 1.),
-                    opacity=.2, align='left')
+                    size=(300, 200),
+                    position=(1615, 875),
+                    color=(0.8, 0.8, 1.0),
+                    opacity=0.2,
+                    align="left",
+                )
 
-                self.help_panel.add_element(text_block, coords=(.02, .01))
+                self.help_panel.add_element(text_block.obj, coords=(0.02, 0.01))
                 scene.add(self.help_panel)
-                self.__tabs.append(ClustersTab(
-                    self.__clusters_visualizer, self.cluster_thr))
+                self.__tabs.append(
+                    ClustersTab(self.__clusters_visualizer, self.cluster_thr)
+                )
 
         sync_slices = sync_vol = False
         self.images = check_img_dtype(self.images)
@@ -460,32 +525,36 @@ class Horizon:
             img_count = 0
             sync_slices, sync_vol = check_img_shapes(self.images)
             for img in self.images:
-                title = 'Image {}'.format(img_count+1)
+                title = f"Image {img_count + 1}"
                 data, affine, fname = unpack_image(img)
                 self.vox2ras = affine
-                if is_binary_image(data):
-                    if self.__roi_images:
-                        if 'rois' in self.random_colors:
-                            roi_color = next(self.color_gen)
-                        roi_actor = actor.contour_from_roi(
-                            data, affine=affine, color=roi_color)
-                        scene.add(roi_actor)
-                        roi_actors.append(roi_actor)
-                    else:
-                        slices_viz = SlicesVisualizer(
-                            self.show_m.iren, scene, data, affine=affine,
-                            world_coords=self.world_coords,
-                            percentiles=[0, 100], rgb=self.rgb)
-                        self.__tabs.append(SlicesTab(
-                            slices_viz, title, fname, self._show_force_render))
-                        img_count += 1
+                if self.__roi_images:
+                    if "rois" in self.random_colors:
+                        roi_color = next(self.color_gen)
+                    roi_actor = actor.contour_from_roi(
+                        data, affine=affine, color=roi_color
+                    )
+                    scene.add(roi_actor)
+                    roi_actors.append(roi_actor)
                 else:
                     slices_viz = SlicesVisualizer(
-                        self.show_m.iren, scene, data, affine=affine,
-                        world_coords=self.world_coords, rgb=self.rgb)
-                    self.__tabs.append(SlicesTab(
-                        slices_viz, title, fname, self._show_force_render))
+                        self.show_m.iren,
+                        scene,
+                        data,
+                        affine=affine,
+                        world_coords=self.world_coords,
+                        rgb=self.rgb,
+                    )
+                    self.__tabs.append(
+                        SlicesTab(
+                            slices_viz,
+                            title,
+                            fname,
+                            force_render=self._show_force_render,
+                        )
+                    )
                     img_count += 1
+
             if len(roi_actors) > 0:
                 self.__tabs.append(ROIsTab(roi_actors))
 
@@ -493,12 +562,16 @@ class Horizon:
         if len(self.pams) > 0:
             if self.images:
                 sync_peaks = check_peak_size(
-                    self.pams, self.images[0][0].shape[:3], sync_slices)
+                    self.pams,
+                    ref_img_shape=self.images[0][0].shape[:3],
+                    sync_imgs=sync_slices,
+                )
             else:
                 sync_peaks = check_peak_size(self.pams)
             for pam in self.pams:
-                peak_viz = PeaksVisualizer((pam.peak_dirs, pam.affine),
-                                           self.world_coords)
+                peak_viz = PeaksVisualizer(
+                    (pam.peak_dirs, pam.affine), self.world_coords
+                )
                 scene.add(peak_viz.actors[0])
                 self.__tabs.append(PeaksTab(peak_viz.actors[0]))
 
@@ -507,10 +580,10 @@ class Horizon:
                 try:
                     vertices, faces, fname = unpack_surface(surface)
                 except ValueError as e:
-                    warn(str(e))
+                    warn(str(e), stacklevel=2)
                     continue
                 color = next(self.color_gen)
-                title = 'Surface {}'.format(idx+1)
+                title = f"Surface {idx + 1}"
                 surf_viz = SurfaceVisualizer((vertices, faces), scene, color)
                 surf_tab = SurfaceTab(surf_viz, title, fname)
                 self.__tabs.append(surf_tab)
@@ -518,43 +591,55 @@ class Horizon:
         self.__win_size = scene.GetSize()
 
         if len(self.__tabs) > 0:
-            def on_tab_changed(actors):
-                for act in actors:
-                    scene.rm(act)
-                    scene.add(act)
-
             self.__tab_mgr = TabManager(
-                self.__tabs, scene.GetSize(),
-                on_tab_changed, sync_slices, sync_vol, sync_peaks)
+                tabs=self.__tabs,
+                win_size=scene.GetSize(),
+                on_tab_changed=self._update_actors,
+                add_to_scene=self._scene.add,
+                remove_from_scene=self._scene.rm,
+                sync_slices=sync_slices,
+                sync_volumes=sync_vol,
+                sync_peaks=sync_peaks,
+            )
 
             scene.add(self.__tab_mgr.tab_ui)
             self.__tab_mgr.handle_text_overflows()
+            self.__tabs[-1].on_tab_selected()
 
         self.show_m.initialize()
 
-        options = [r'un\hide centroids', 'invert selection',
-                   r'un\select all', 'expand clusters',
-                   'collapse clusters', 'save streamlines',
-                   'recluster']
-        listbox = ui.ListBox2D(values=options, position=(10, 300),
-                               size=(200, 270),
-                               multiselection=False, font_size=18)
+        options = [
+            r"un\hide centroids",
+            "invert selection",
+            r"un\select all",
+            "expand clusters",
+            "collapse clusters",
+            "save streamlines",
+            "recluster",
+        ]
+        listbox = ui.ListBox2D(
+            values=options,
+            position=(10, 300),
+            size=(200, 270),
+            multiselection=False,
+            font_size=18,
+        )
 
         def display_element():
             action = listbox.selected[0]
-            if action == r'un\hide centroids':
+            if action == r"un\hide centroids":
                 self.__hide()
-            if action == 'invert selection':
+            if action == "invert selection":
                 self.__invert()
-            if action == r'un\select all':
+            if action == r"un\select all":
                 self.__show_all()
-            if action == 'expand clusters':
+            if action == "expand clusters":
                 self.__expand()
-            if action == 'collapse clusters':
+            if action == "collapse clusters":
                 self.__reset()
-            if action == 'save streamlines':
+            if action == "save streamlines":
                 self.__save()
-            if action == 'recluster':
+            if action == "recluster":
                 self.__new_window()
 
         listbox.on_change = display_element
@@ -564,10 +649,10 @@ class Horizon:
         self.show_m.scene.add(listbox)
 
         def left_click_centroid_callback(obj, event):
-
-            self.cea[obj]['selected'] = not self.cea[obj]['selected']
-            self.cla[self.cea[obj]['cluster_actor']]['selected'] = \
-                self.cea[obj]['selected']
+            self.cea[obj]["selected"] = not self.cea[obj]["selected"]
+            self.cla[self.cea[obj]["cluster_actor"]]["selected"] = self.cea[obj][
+                "selected"
+            ]
             self.show_m.render()
 
         def right_click_centroid_callback(obj, event):
@@ -578,36 +663,32 @@ class Horizon:
             self.show_m.render()
 
         def left_click_cluster_callback(obj, event):
-
-            if self.cla[obj]['selected']:
-                self.cla[obj]['centroid_actor'].VisibilityOn()
-                ca = self.cla[obj]['centroid_actor']
-                self.cea[ca]['selected'] = 0
+            if self.cla[obj]["selected"]:
+                self.cla[obj]["centroid_actor"].VisibilityOn()
+                ca = self.cla[obj]["centroid_actor"]
+                self.cea[ca]["selected"] = 0
                 obj.VisibilityOff()
-                self.cea[ca]['expanded'] = 0
+                self.cea[ca]["expanded"] = 0
 
             self.show_m.render()
 
         def right_click_cluster_callback(obj, event):
-            print('Cluster Area Selected')
+            print("Cluster Area Selected")
             self.show_m.render()
 
         for cl in self.cla:
-            cl.AddObserver('LeftButtonPressEvent',
-                           left_click_cluster_callback,
-                           1.0)
-            cl.AddObserver('RightButtonPressEvent',
-                           right_click_cluster_callback,
-                           1.0)
-            self.cla[cl]['centroid_actor'].AddObserver(
-                'LeftButtonPressEvent', left_click_centroid_callback, 1.0)
-            self.cla[cl]['centroid_actor'].AddObserver(
-                'RightButtonPressEvent', right_click_centroid_callback, 1.0)
+            cl.AddObserver("LeftButtonPressEvent", left_click_cluster_callback, 1.0)
+            cl.AddObserver("RightButtonPressEvent", right_click_cluster_callback, 1.0)
+            self.cla[cl]["centroid_actor"].AddObserver(
+                "LeftButtonPressEvent", left_click_centroid_callback, 1.0
+            )
+            self.cla[cl]["centroid_actor"].AddObserver(
+                "RightButtonPressEvent", right_click_centroid_callback, 1.0
+            )
 
         self.mem.window_timer_cnt = 0
 
         def timer_callback(obj, event):
-
             self.mem.window_timer_cnt += 1
             # TODO possibly add automatic rotation option
             # self.show_m.scene.azimuth(0.01 * self.mem.window_timer_cnt)
@@ -618,11 +699,9 @@ class Horizon:
         scene.reset_clipping_range()
 
         if self.interactive:
-
             self.show_m.add_window_callback(self.__win_callback)
             self.show_m.add_timer_callback(True, 200, timer_callback)
-            self.show_m.iren.AddObserver(
-                'KeyPressEvent', self.__key_press_events)
+            self.show_m.iren.AddObserver("KeyPressEvent", self.__key_press_events)
 
             if self.return_showm:
                 return self.show_m
@@ -632,7 +711,6 @@ class Horizon:
                 self.show_m.start()
 
             else:
-
                 # set to True if event recorded file needs updating
                 recording = False
                 recording_filename = self.recorded_events
@@ -643,27 +721,47 @@ class Horizon:
                     self.show_m.play_events_from_file(recording_filename)
 
         else:
-
-            window.record(scene, out_path=self.out_png,
-                          size=(1200, 900),
-                          reset_camera=False)
+            window.record(
+                scene=scene, out_path=self.out_png, size=(1200, 900), reset_camera=False
+            )
 
 
-def horizon(tractograms=None, images=None, pams=None, surfaces=None,
-            cluster=False, rgb=False, cluster_thr=15.0,
-            random_colors=None, bg_color=(0, 0, 0), order_transparent=True,
-            length_gt=0, length_lt=1000, clusters_gt=0, clusters_lt=10000,
-            world_coords=True, interactive=True, buan=False, buan_colors=None,
-            roi_images=False, roi_colors=(1, 0, 0), out_png='tmp.png',
-            recorded_events=None, return_showm=False):
+@warning_for_keywords()
+def horizon(
+    *,
+    tractograms=None,
+    images=None,
+    pams=None,
+    surfaces=None,
+    cluster=False,
+    rgb=False,
+    cluster_thr=15.0,
+    random_colors=None,
+    bg_color=(0, 0, 0),
+    order_transparent=True,
+    length_gt=0,
+    length_lt=1000,
+    clusters_gt=0,
+    clusters_lt=10000,
+    world_coords=True,
+    interactive=True,
+    buan=False,
+    buan_colors=None,
+    roi_images=False,
+    roi_colors=(1, 0, 0),
+    out_png="tmp.png",
+    recorded_events=None,
+    return_showm=False,
+):
     """Interactive medical visualization - Invert the Horizon!
 
+    See :footcite:p:`Garyfallidis2019` for further details about Horizon.
 
     Parameters
     ----------
     tractograms : sequence of StatefulTractograms
-            StatefulTractograms are used for making sure that the coordinate
-            systems are correct
+        StatefulTractograms are used for making sure that the coordinate
+        systems are correct
     images : sequence of tuples
         Each tuple contains data and affine
     pams : sequence of PeakAndMetrics
@@ -721,25 +819,39 @@ def horizon(tractograms=None, images=None, pams=None, surfaces=None,
         File path to replay recorded events
     return_showm : bool
         Return ShowManager object. Used only at Python level. Can be used
-        for extending Horizon's cababilities externally and for testing
+        for extending Horizon's capabilities externally and for testing
         purposes.
 
     References
     ----------
-    .. [Horizon_ISMRM19] Garyfallidis E., M-A. Cote, B.Q. Chandio,
-        S. Fadnavis, J. Guaje, R. Aggarwal, E. St-Onge, K.S. Juneja,
-        S. Koudoro, D. Reagan, DIPY Horizon: fast, modular, unified and
-        adaptive visualization, Proceedings of: International Society of
-        Magnetic Resonance in Medicine (ISMRM), Montreal, Canada, 2019.
+    .. footbibliography::
     """
 
-    hz = Horizon(tractograms, images, pams, surfaces, cluster, rgb,
-                 cluster_thr, random_colors, length_gt, length_lt, clusters_gt,
-                 clusters_lt, world_coords, interactive, out_png,
-                 recorded_events, return_showm, bg_color=bg_color,
-                 order_transparent=order_transparent, buan=buan,
-                 buan_colors=buan_colors, roi_images=roi_images,
-                 roi_colors=roi_colors)
+    hz = Horizon(
+        tractograms=tractograms,
+        images=images,
+        pams=pams,
+        surfaces=surfaces,
+        cluster=cluster,
+        rgb=rgb,
+        cluster_thr=cluster_thr,
+        random_colors=random_colors,
+        length_gt=length_gt,
+        length_lt=length_lt,
+        clusters_gt=clusters_gt,
+        clusters_lt=clusters_lt,
+        world_coords=world_coords,
+        interactive=interactive,
+        out_png=out_png,
+        recorded_events=recorded_events,
+        return_showm=return_showm,
+        bg_color=bg_color,
+        order_transparent=order_transparent,
+        buan=buan,
+        buan_colors=buan_colors,
+        roi_images=roi_images,
+        roi_colors=roi_colors,
+    )
 
     scene = hz.build_scene()
 
