@@ -1,33 +1,49 @@
 import logging
 import shutil
+
 import numpy as np
 
 from dipy.core.gradients import gradient_table
+from dipy.denoise.gibbs import gibbs_removal
+from dipy.denoise.localpca import localpca, mppca
+from dipy.denoise.nlmeans import nlmeans
+from dipy.denoise.noise_estimate import estimate_sigma
+from dipy.denoise.patch2self import patch2self
+from dipy.denoise.pca_noise_estimate import pca_noise_estimate
 from dipy.io.gradients import read_bvals_bvecs
 from dipy.io.image import load_nifti, save_nifti
-from dipy.denoise.patch2self import patch2self
-from dipy.denoise.nlmeans import nlmeans
-from dipy.denoise.localpca import localpca, mppca
-from dipy.denoise.gibbs import gibbs_removal
-from dipy.denoise.noise_estimate import estimate_sigma
-from dipy.denoise.pca_noise_estimate import pca_noise_estimate
 from dipy.workflows.workflow import Workflow
 
 
 class Patch2SelfFlow(Workflow):
     @classmethod
     def get_short_name(cls):
-        return 'patch2self'
+        return "patch2self"
 
-    def run(self, input_files, bval_files, model='ols',
-            b0_threshold=50, alpha=1.0, verbose=False, patch_radius=0,
-            b0_denoising=True, clip_negative_vals=False, shift_intensity=True,
-            out_dir='', out_denoised='dwi_patch2self.nii.gz'):
+    def run(
+        self,
+        input_files,
+        bval_files,
+        model="ols",
+        b0_threshold=50,
+        alpha=1.0,
+        verbose=False,
+        patch_radius=0,
+        b0_denoising=True,
+        clip_negative_vals=False,
+        shift_intensity=True,
+        ver=3,
+        out_dir="",
+        out_denoised="dwi_patch2self.nii.gz",
+    ):
         """Workflow for Patch2Self denoising method.
 
-        It applies patch2self denoising on each file found by 'globing'
-        ``input_file`` and ``bval_file``. It saves the results in a directory
-        specified by ``out_dir``.
+        See :footcite:p:`Fadnavis2020` for further details about the method.
+        See :footcite:p:`Fadnavis2024` for further details about the new method.
+
+        It applies patch2self denoising :footcite:p:`Fadnavis2020` on each file
+        found by 'globing' ``input_file`` and ``bval_file``. It saves the
+        results in a directory specified by ``out_dir``.
 
         Parameters
         ----------
@@ -62,6 +78,8 @@ class Patch2SelfFlow(Workflow):
         shift_intensity : bool, optional
             Shifts the distribution of intensities per volume to give
             non-negative values
+        ver : int, optional
+            Version of the Patch2Self algorithm to use between  1 or 3.
         out_dir : string, optional
             Output directory (default current directory)
         out_denoised : string, optional
@@ -70,9 +88,7 @@ class Patch2SelfFlow(Workflow):
 
         References
         ----------
-        .. [Fadnavis20] S. Fadnavis, J. Batson, E. Garyfallidis, Patch2Self:
-                    Denoising Diffusion MRI with Self-supervised Learning,
-                    Advances in Neural Information Processing Systems 33 (2020)
+        .. footbibliography::
 
         """
         io_it = self.get_io_iterator()
@@ -81,36 +97,50 @@ class Patch2SelfFlow(Workflow):
         for fpath, bvalpath, odenoised in io_it:
             if self._skip:
                 shutil.copy(fpath, odenoised)
-                logging.warning('Denoising skipped for now.')
+                logging.warning("Denoising skipped for now.")
             else:
-                logging.info('Denoising %s', fpath)
+                logging.info("Denoising %s", fpath)
                 data, affine, image = load_nifti(fpath, return_img=True)
                 bvals = np.loadtxt(bvalpath)
-
+                extra_args = {"patch_radius": patch_radius} if ver == 1 else {}
                 denoised_data = patch2self(
-                    data, bvals, model=model, b0_threshold=b0_threshold,
-                    alpha=alpha, verbose=verbose, patch_radius=patch_radius,
+                    data,
+                    bvals,
+                    model=model,
+                    b0_threshold=b0_threshold,
+                    alpha=alpha,
+                    verbose=verbose,
                     b0_denoising=b0_denoising,
                     clip_negative_vals=clip_negative_vals,
                     shift_intensity=shift_intensity,
+                    version=ver,
+                    **extra_args,
                 )
-                save_nifti(odenoised, denoised_data, affine, image.header)
+                save_nifti(odenoised, denoised_data, affine, hdr=image.header)
 
-                logging.info('Denoised volumes saved as %s', odenoised)
+                logging.info("Denoised volumes saved as %s", odenoised)
 
 
 class NLMeansFlow(Workflow):
     @classmethod
     def get_short_name(cls):
-        return 'nlmeans'
+        return "nlmeans"
 
-    def run(self, input_files, sigma=0, patch_radius=1, block_radius=5,
-            rician=True, out_dir='', out_denoised='dwi_nlmeans.nii.gz'):
+    def run(
+        self,
+        input_files,
+        sigma=0,
+        patch_radius=1,
+        block_radius=5,
+        rician=True,
+        out_dir="",
+        out_denoised="dwi_nlmeans.nii.gz",
+    ):
         """Workflow wrapping the nlmeans denoising method.
 
-        It applies nlmeans denoise on each file found by 'globing'
-        ``input_files`` and saves the results in a directory specified by
-        ``out_dir``.
+        It applies nlmeans denoise :footcite:p:`Descoteaux2008a` on each file
+        found by 'globing' ``input_files`` and saves the results in a directory
+        specified by ``out_dir``.
 
         Parameters
         ----------
@@ -133,45 +163,58 @@ class NLMeansFlow(Workflow):
 
         References
         ----------
-        .. [Descoteaux08] Descoteaux, Maxime and Wiest-Daesslé, Nicolas and
-        Prima, Sylvain and Barillot, Christian and Deriche, Rachid.
-        Impact of Rician Adapted Non-Local Means Filtering on
-        HARDI, MICCAI 2008
+        .. footbibliography::
 
         """
         io_it = self.get_io_iterator()
         for fpath, odenoised in io_it:
             if self._skip:
                 shutil.copy(fpath, odenoised)
-                logging.warning('Denoising skipped for now.')
+                logging.warning("Denoising skipped for now.")
             else:
-                logging.info('Denoising %s', fpath)
+                logging.info("Denoising %s", fpath)
                 data, affine, image = load_nifti(fpath, return_img=True)
 
                 if sigma == 0:
-                    logging.info('Estimating sigma')
+                    logging.info("Estimating sigma")
                     sigma = estimate_sigma(data)
-                    logging.debug('Found sigma {0}'.format(sigma))
+                    logging.debug(f"Found sigma {sigma}")
 
-                denoised_data = nlmeans(data, sigma=sigma,
-                                        patch_radius=patch_radius,
-                                        block_radius=block_radius,
-                                        rician=rician)
-                save_nifti(odenoised, denoised_data, affine, image.header)
+                denoised_data = nlmeans(
+                    data,
+                    sigma=sigma,
+                    patch_radius=patch_radius,
+                    block_radius=block_radius,
+                    rician=rician,
+                )
+                save_nifti(odenoised, denoised_data, affine, hdr=image.header)
 
-                logging.info('Denoised volume saved as %s', odenoised)
+                logging.info("Denoised volume saved as %s", odenoised)
 
 
 class LPCAFlow(Workflow):
     @classmethod
     def get_short_name(cls):
-        return 'lpca'
+        return "lpca"
 
-    def run(self, input_files, bvalues_files, bvectors_files, sigma=0,
-            b0_threshold=50, bvecs_tol=0.01, patch_radius=2, pca_method='eig',
-            tau_factor=2.3, out_dir='', out_denoised='dwi_lpca.nii.gz'):
+    def run(
+        self,
+        input_files,
+        bvalues_files,
+        bvectors_files,
+        sigma=0,
+        b0_threshold=50,
+        bvecs_tol=0.01,
+        patch_radius=2,
+        pca_method="eig",
+        tau_factor=2.3,
+        out_dir="",
+        out_denoised="dwi_lpca.nii.gz",
+    ):
         r"""Workflow wrapping LPCA denoising method.
 
+        See :footcite:p:`Veraart2016a` for further details about the method.
+
         Parameters
         ----------
         input_files : string
@@ -185,8 +228,8 @@ class LPCAFlow(Workflow):
             multiple bvectors files at once.
         sigma : float, optional
             Standard deviation of the noise estimated from the data.
-            Default 0: it means sigma value estimation with the Manjon2013
-            algorithm [3]_.
+            0 means sigma value estimation following the algorithm in
+            :footcite:t:`Manjon2013`.
         b0_threshold : float, optional
             Threshold used to find b0 volumes.
         bvecs_tol : float, optional
@@ -206,14 +249,14 @@ class LPCAFlow(Workflow):
             Thresholding of PCA eigenvalues is done by nulling out eigenvalues
             that are smaller than:
 
-            .. math ::
+            .. math::
 
                     \tau = (\tau_{factor} \sigma)^2
 
-            \tau_{factor} can be change to adjust the relationship between the
-            noise standard deviation and the threshold \tau. If \tau_{factor}
-            is set to None, it will be automatically calculated using the
-            Marcenko-Pastur distribution [2]_.
+            $\tau_{factor}$ can be change to adjust the relationship between the
+            noise standard deviation and the threshold $\tau$. If
+            $\tau_{factor}$ is set to None, it will be automatically calculated
+            using the Marcenko-Pastur distribution :footcite:p`Veraart2016b`.
         out_dir : string, optional
             Output directory. (default current directory)
         out_denoised : string, optional
@@ -221,57 +264,56 @@ class LPCAFlow(Workflow):
 
         References
         ----------
-        .. [1] Veraart J, Novikov DS, Christiaens D, Ades-aron B, Sijbers,
-        Fieremans E, 2016. Denoising of Diffusion MRI using random
-        matrix theory. Neuroimage 142:394-406.
-        doi: 10.1016/j.neuroimage.2016.08.016
-
-        .. [2] Veraart J, Fieremans E, Novikov DS. 2016. Diffusion MRI noise
-        mapping using random matrix theory. Magnetic Resonance in
-        Medicine.
-        doi: 10.1002/mrm.26059.
-
-        .. [3] Manjon JV, Coupe P, Concha L, Buades A, Collins DL (2013)
-        Diffusion Weighted Image Denoising Using Overcomplete Local
-        PCA. PLoS ONE 8(9): e73021.
-        https://doi.org/10.1371/journal.pone.0073021
+        .. footbibliography::
 
         """
         io_it = self.get_io_iterator()
         if isinstance(patch_radius, list) and len(patch_radius) == 1:
             patch_radius = int(patch_radius[0])
         for dwi, bval, bvec, odenoised in io_it:
-            logging.info('Denoising %s', dwi)
+            logging.info("Denoising %s", dwi)
             data, affine, image = load_nifti(dwi, return_img=True)
 
             if not sigma:
-                logging.info('Estimating sigma')
+                logging.info("Estimating sigma")
                 bvals, bvecs = read_bvals_bvecs(bval, bvec)
-                gtab = gradient_table(bvals, bvecs, b0_threshold=b0_threshold,
-                                      atol=bvecs_tol)
-                sigma = pca_noise_estimate(data, gtab, correct_bias=True,
-                                           smooth=3)
-                logging.debug('Found sigma %s', sigma)
+                gtab = gradient_table(
+                    bvals, bvecs=bvecs, b0_threshold=b0_threshold, atol=bvecs_tol
+                )
+                sigma = pca_noise_estimate(data, gtab, correct_bias=True, smooth=3)
+                logging.debug("Found sigma %s", sigma)
 
-            denoised_data = localpca(data, sigma=sigma,
-                                     patch_radius=patch_radius,
-                                     pca_method=pca_method,
-                                     tau_factor=tau_factor)
-            save_nifti(odenoised, denoised_data, affine, image.header)
+            denoised_data = localpca(
+                data,
+                sigma=sigma,
+                patch_radius=patch_radius,
+                pca_method=pca_method,
+                tau_factor=tau_factor,
+            )
+            save_nifti(odenoised, denoised_data, affine, hdr=image.header)
 
-            logging.info('Denoised volume saved as %s', odenoised)
+            logging.info("Denoised volume saved as %s", odenoised)
 
 
 class MPPCAFlow(Workflow):
     @classmethod
     def get_short_name(cls):
-        return 'mppca'
+        return "mppca"
 
-    def run(self, input_files, patch_radius=2, pca_method='eig',
-            return_sigma=False, out_dir='', out_denoised='dwi_mppca.nii.gz',
-            out_sigma='dwi_sigma.nii.gz'):
+    def run(
+        self,
+        input_files,
+        patch_radius=2,
+        pca_method="eig",
+        return_sigma=False,
+        out_dir="",
+        out_denoised="dwi_mppca.nii.gz",
+        out_sigma="dwi_sigma.nii.gz",
+    ):
         r"""Workflow wrapping Marcenko-Pastur PCA denoising method.
 
+        See :footcite:p:`Veraart2016a` for further details about the method.
+
         Parameters
         ----------
         input_files : string
@@ -289,7 +331,7 @@ class MPPCAFlow(Workflow):
             be more accurate.
         return_sigma : bool, optional
             If true, a noise standard deviation estimate based on the
-            Marcenko-Pastur distribution is returned [2]_.
+            Marcenko-Pastur distribution is returned :footcite:p:`Veraart2016b`.
         out_dir : string, optional
             Output directory. (default current directory)
         out_denoised : string, optional
@@ -299,14 +341,7 @@ class MPPCAFlow(Workflow):
 
         References
         ----------
-        .. [1] Veraart J, Novikov DS, Christiaens D, Ades-aron B, Sijbers,
-        Fieremans E, 2016. Denoising of Diffusion MRI using random matrix
-        theory. Neuroimage 142:394-406.
-        doi: 10.1016/j.neuroimage.2016.08.016
-
-        .. [2] Veraart J, Fieremans E, Novikov DS. 2016. Diffusion MRI noise
-        mapping using random matrix theory. Magnetic Resonance in Medicine.
-        doi: 10.1002/mrm.26059.
+        .. footbibliography::
 
         """
         io_it = self.get_io_iterator()
@@ -314,29 +349,42 @@ class MPPCAFlow(Workflow):
             patch_radius = int(patch_radius[0])
 
         for dwi, odenoised, osigma in io_it:
-            logging.info('Denoising %s', dwi)
+            logging.info("Denoising %s", dwi)
             data, affine, image = load_nifti(dwi, return_img=True)
 
-            denoised_data, sigma = mppca(data, patch_radius=patch_radius,
-                                         pca_method=pca_method,
-                                         return_sigma=True)
+            denoised_data, sigma = mppca(
+                data,
+                patch_radius=patch_radius,
+                pca_method=pca_method,
+                return_sigma=True,
+            )
 
-            save_nifti(odenoised, denoised_data, affine, image.header)
-            logging.info('Denoised volume saved as %s', odenoised)
+            save_nifti(odenoised, denoised_data, affine, hdr=image.header)
+            logging.info("Denoised volume saved as %s", odenoised)
             if return_sigma:
-                save_nifti(osigma, sigma, affine, image.header)
-                logging.info('Sigma volume saved as %s', osigma)
+                save_nifti(osigma, sigma, affine, hdr=image.header)
+                logging.info("Sigma volume saved as %s", osigma)
 
 
 class GibbsRingingFlow(Workflow):
     @classmethod
     def get_short_name(cls):
-        return 'gibbs_ringing'
+        return "gibbs_ringing"
 
-    def run(self, input_files, slice_axis=2, n_points=3, num_processes=1,
-            out_dir='', out_unring='dwi_unring.nii.gz'):
+    def run(
+        self,
+        input_files,
+        slice_axis=2,
+        n_points=3,
+        num_processes=1,
+        out_dir="",
+        out_unring="dwi_unring.nii.gz",
+    ):
         r"""Workflow for applying Gibbs Ringing method.
 
+        See :footcite:p:`Kellner2016` and :footcite:p:`NetoHenriques2018` for
+        further details about the method.
+
         Parameters
         ----------
         input_files : string
@@ -360,23 +408,20 @@ class GibbsRingingFlow(Workflow):
 
         References
         ----------
-        .. [1] Neto Henriques, R., 2018. Advanced Methods for Diffusion MRI
-        Data Analysis and their Application to the Healthy Ageing Brain
-        (Doctoral thesis). https://doi.org/10.17863/CAM.29356
-
-        .. [2] Kellner E, Dhital B, Kiselev VG, Reisert M. Gibbs-ringing
-        artifact removal based on local subvoxel-shifts. Magn Reson Med. 2016
-        doi: 10.1002/mrm.26054.
+        .. footbibliography::
 
         """
         io_it = self.get_io_iterator()
         for dwi, ounring in io_it:
-            logging.info('Unringing %s', dwi)
+            logging.info("Unringing %s", dwi)
             data, affine, image = load_nifti(dwi, return_img=True)
 
-            unring_data = gibbs_removal(data, slice_axis=slice_axis,
-                                        n_points=n_points,
-                                        num_processes=num_processes)
+            unring_data = gibbs_removal(
+                data,
+                slice_axis=slice_axis,
+                n_points=n_points,
+                num_processes=num_processes,
+            )
 
-            save_nifti(ounring, unring_data, affine, image.header)
-            logging.info('Denoised volume saved as %s', ounring)
+            save_nifti(ounring, unring_data, affine, hdr=image.header)
+            logging.info("Denoised volume saved as %s", ounring)
