@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 
 import logging
+
 import numpy as np
 
 from dipy.io.image import load_nifti, save_nifti
@@ -10,12 +11,10 @@ from dipy.workflows.workflow import Workflow
 class MaskFlow(Workflow):
     @classmethod
     def get_short_name(cls):
-        return 'mask'
+        return "mask"
 
-    def run(self, input_files, lb, ub=np.inf, out_dir='',
-            out_mask='mask.nii.gz'):
-
-        """ Workflow for creating a binary mask
+    def run(self, input_files, lb, ub=np.inf, out_dir="", out_mask="mask.nii.gz"):
+        """Workflow for creating a binary mask
 
         Parameters
         ----------
@@ -31,15 +30,17 @@ class MaskFlow(Workflow):
            Name of the masked file.
         """
         if lb >= ub:
-            logging.error('The upper bound(less than) should be greater'
-                          ' than the lower bound (greater_than).')
+            logging.error(
+                "The upper bound(less than) should be greater"
+                " than the lower bound (greater_than)."
+            )
             return
 
         io_it = self.get_io_iterator()
 
         for input_path, out_mask_path in io_it:
-            logging.info('Creating mask of {0}'.format(input_path))
+            logging.info(f"Creating mask of {input_path}")
             data, affine = load_nifti(input_path)
             mask = np.bitwise_and(data > lb, data < ub)
             save_nifti(out_mask_path, mask.astype(np.ubyte), affine)
-            logging.info('Mask saved at {0}'.format(out_mask_path))
+            logging.info(f"Mask saved at {out_mask_path}")
