@@ -3,8 +3,8 @@
 Automatic Fiber Bundle Extraction with RecoBundles
 ==================================================
 
-This example explains how we can use RecoBundles [Garyfallidis17]_ to extract
-bundles from tractograms.
+This example explains how we can use RecoBundles :footcite:p:`Garyfallidis2018`
+to extract bundles from tractograms.
 
 First import the necessary modules.
 """
@@ -12,11 +12,13 @@ First import the necessary modules.
 import numpy as np
 
 from dipy.align.streamlinear import whole_brain_slr
-from dipy.data import get_two_hcp842_bundles
-from dipy.data import (fetch_target_tractogram_hcp,
-                       fetch_bundle_atlas_hcp842,
-                       get_bundle_atlas_hcp842,
-                       get_target_tractogram_hcp)
+from dipy.data import (
+    fetch_bundle_atlas_hcp842,
+    fetch_target_tractogram_hcp,
+    get_bundle_atlas_hcp842,
+    get_target_tractogram_hcp,
+    get_two_hcp842_bundles,
+)
 from dipy.io.stateful_tractogram import Space, StatefulTractogram
 from dipy.io.streamline import load_trk, save_trk
 from dipy.io.utils import create_tractogram_header
@@ -34,13 +36,11 @@ target_file = get_target_tractogram_hcp()
 
 sft_atlas = load_trk(atlas_file, "same", bbox_valid_check=False)
 atlas = sft_atlas.streamlines
-atlas_header = create_tractogram_header(atlas_file,
-                                        *sft_atlas.space_attributes)
+atlas_header = create_tractogram_header(atlas_file, *sft_atlas.space_attributes)
 
 sft_target = load_trk(target_file, "same", bbox_valid_check=False)
 target = sft_target.streamlines
-target_header = create_tractogram_header(target_file,
-                                         *sft_target.space_attributes)
+target_header = create_tractogram_header(target_file, *sft_target.space_attributes)
 
 ###############################################################################
 # let's visualize atlas tractogram and target tractogram before registration
@@ -51,7 +51,7 @@ scene = window.Scene()
 scene.SetBackground(1, 1, 1)
 scene.add(actor.line(atlas, colors=(1, 0, 1)))
 scene.add(actor.line(target, colors=(1, 1, 0)))
-window.record(scene, out_path='tractograms_initial.png', size=(600, 600))
+window.record(scene=scene, out_path="tractograms_initial.png", size=(600, 600))
 if interactive:
     window.show(scene)
 
@@ -62,11 +62,16 @@ if interactive:
 #
 #
 # We will register target tractogram to model atlas' space using streamlinear
-# registration (SLR) [Garyfallidis15]_
+# registration (SLR) :footcite:p:`Garyfallidis2015`.
 
 moved, transform, qb_centroids1, qb_centroids2 = whole_brain_slr(
-    atlas, target, x0='affine', verbose=True, progressive=True,
-    rng=np.random.RandomState(1984))
+    atlas,
+    target,
+    x0="affine",
+    verbose=True,
+    progressive=True,
+    rng=np.random.RandomState(1984),
+)
 
 
 ###############################################################################
@@ -84,8 +89,9 @@ scene = window.Scene()
 scene.SetBackground(1, 1, 1)
 scene.add(actor.line(atlas, colors=(1, 0, 1)))
 scene.add(actor.line(moved, colors=(1, 1, 0)))
-window.record(scene, out_path='tractograms_after_registration.png',
-              size=(600, 600))
+window.record(
+    scene=scene, out_path="tractograms_after_registration.png", size=(600, 600)
+)
 if interactive:
     window.show(scene)
 
@@ -96,7 +102,7 @@ if interactive:
 #
 #
 #
-# Extracting bundles using RecoBundles [Garyfallidis17]_
+# Extracting bundles using RecoBundles :footcite:p:`Garyfallidis2018`
 #
 # RecoBundles requires a model (reference) bundle and tries to extract similar
 # looking bundle from the input tractogram. There are some key parameters that
@@ -138,7 +144,7 @@ if interactive:
 # Arcuate Fasciculus Left model bundle.
 
 model_af_l_file, model_cst_l_file = get_two_hcp842_bundles()
-sft_af_l = load_trk(model_af_l_file, "same", bbox_valid_check=False)
+sft_af_l = load_trk(model_af_l_file, reference="same", bbox_valid_check=False)
 model_af_l = sft_af_l.streamlines
 
 interactive = False
@@ -146,11 +152,12 @@ interactive = False
 scene = window.Scene()
 scene.SetBackground(1, 1, 1)
 scene.add(actor.line(model_af_l))
-scene.set_camera(focal_point=(-18.17281532, -19.55606842, 6.92485857),
-                 position=(-360.11, -30.46, -40.44),
-                 view_up=(-0.03, 0.028, 0.89))
-window.record(scene, out_path='AF_L_model_bundle.png',
-              size=(600, 600))
+scene.set_camera(
+    focal_point=(-18.17281532, -19.55606842, 6.92485857),
+    position=(-360.11, -30.46, -40.44),
+    view_up=(-0.03, 0.028, 0.89),
+)
+window.record(scene=scene, out_path="AF_L_model_bundle.png", size=(600, 600))
 if interactive:
     window.show(scene)
 
@@ -161,13 +168,15 @@ if interactive:
 
 rb = RecoBundles(moved, verbose=True, rng=np.random.RandomState(2001))
 
-recognized_af_l, af_l_labels = rb.recognize(model_bundle=model_af_l,
-                                            model_clust_thr=0.1,
-                                            reduction_thr=15,
-                                            pruning_thr=7,
-                                            reduction_distance='mdf',
-                                            pruning_distance='mdf',
-                                            slr=True)
+recognized_af_l, af_l_labels = rb.recognize(
+    model_bundle=model_af_l,
+    model_clust_thr=0.1,
+    reduction_thr=15,
+    pruning_thr=7,
+    reduction_distance="mdf",
+    pruning_distance="mdf",
+    slr=True,
+)
 
 ###############################################################################
 # let's visualize extracted Arcuate Fasciculus Left bundle
@@ -177,11 +186,12 @@ interactive = False
 scene = window.Scene()
 scene.SetBackground(1, 1, 1)
 scene.add(actor.line(recognized_af_l.copy()))
-scene.set_camera(focal_point=(-18.17281532, -19.55606842, 6.92485857),
-                 position=(-360.11, -30.46, -40.44),
-                 view_up=(-0.03, 0.028, 0.89))
-window.record(scene, out_path='AF_L_recognized_bundle.png',
-              size=(600, 600))
+scene.set_camera(
+    focal_point=(-18.17281532, -19.55606842, 6.92485857),
+    position=(-360.11, -30.46, -40.44),
+    view_up=(-0.03, 0.028, 0.89),
+)
+window.record(scene=scene, out_path="AF_L_recognized_bundle.png", size=(600, 600))
 if interactive:
     window.show(scene)
 
@@ -195,28 +205,28 @@ if interactive:
 # Save the bundle as a trk file. Let's save the recognized bundle in the
 # common space (atlas space), in this case, MNI space.
 
-reco_af_l = StatefulTractogram(recognized_af_l, atlas_header,
-                               Space.RASMM)
+reco_af_l = StatefulTractogram(recognized_af_l, atlas_header, Space.RASMM)
 save_trk(reco_af_l, "AF_L_rec_1.trk", bbox_valid_check=False)
 
 ###############################################################################
 # Let's save the recognized bundle in the original space of the subject
 # anatomy.
 
-reco_af_l = StatefulTractogram(target[af_l_labels], target_header,
-                               Space.RASMM)
+reco_af_l = StatefulTractogram(target[af_l_labels], target_header, Space.RASMM)
 save_trk(reco_af_l, "AF_L_org_1.trk", bbox_valid_check=False)
 
 ###############################################################################
 # Now, let's increase the reduction_thr and pruning_thr values.
 
-recognized_af_l, af_l_labels = rb.recognize(model_bundle=model_af_l,
-                                            model_clust_thr=0.1,
-                                            reduction_thr=20,
-                                            pruning_thr=10,
-                                            reduction_distance='mdf',
-                                            pruning_distance='mdf',
-                                            slr=True)
+recognized_af_l, af_l_labels = rb.recognize(
+    model_bundle=model_af_l,
+    model_clust_thr=0.1,
+    reduction_thr=20,
+    pruning_thr=10,
+    reduction_distance="mdf",
+    pruning_distance="mdf",
+    slr=True,
+)
 
 ###############################################################################
 # let's visualize extracted Arcuate Fasciculus Left bundle.
@@ -226,11 +236,12 @@ interactive = False
 scene = window.Scene()
 scene.SetBackground(1, 1, 1)
 scene.add(actor.line(recognized_af_l.copy()))
-scene.set_camera(focal_point=(-18.17281532, -19.55606842, 6.92485857),
-                 position=(-360.11, -30.46, -40.44),
-                 view_up=(-0.03, 0.028, 0.89))
-window.record(scene, out_path='AF_L_recognized_bundle2.png',
-              size=(600, 600))
+scene.set_camera(
+    focal_point=(-18.17281532, -19.55606842, 6.92485857),
+    position=(-360.11, -30.46, -40.44),
+    view_up=(-0.03, 0.028, 0.89),
+)
+window.record(scene=scene, out_path="AF_L_recognized_bundle2.png", size=(600, 600))
 if interactive:
     window.show(scene)
 
@@ -243,28 +254,28 @@ if interactive:
 # Save the bundle as a trk file. Let's save the recognized bundle in the
 # common space (atlas space), in this case, MNI space.
 
-reco_af_l = StatefulTractogram(recognized_af_l, atlas_header,
-                               Space.RASMM)
+reco_af_l = StatefulTractogram(recognized_af_l, atlas_header, Space.RASMM)
 save_trk(reco_af_l, "AF_L_rec_2.trk", bbox_valid_check=False)
 
 ###############################################################################
 # Let's save the recognized bundle in the original space of the subject
 # anatomy.
 
-reco_af_l = StatefulTractogram(target[af_l_labels], target_header,
-                               Space.RASMM)
+reco_af_l = StatefulTractogram(target[af_l_labels], target_header, Space.RASMM)
 save_trk(reco_af_l, "AF_L_org_2.trk", bbox_valid_check=False)
 
 ###############################################################################
 # Now, let's increase the reduction_thr and pruning_thr values further.
 
-recognized_af_l, af_l_labels = rb.recognize(model_bundle=model_af_l,
-                                            model_clust_thr=0.1,
-                                            reduction_thr=25,
-                                            pruning_thr=12,
-                                            reduction_distance='mdf',
-                                            pruning_distance='mdf',
-                                            slr=True)
+recognized_af_l, af_l_labels = rb.recognize(
+    model_bundle=model_af_l,
+    model_clust_thr=0.1,
+    reduction_thr=25,
+    pruning_thr=12,
+    reduction_distance="mdf",
+    pruning_distance="mdf",
+    slr=True,
+)
 
 ###############################################################################
 # let's visualize extracted Arcuate Fasciculus Left bundle.
@@ -274,11 +285,12 @@ interactive = False
 scene = window.Scene()
 scene.SetBackground(1, 1, 1)
 scene.add(actor.line(recognized_af_l.copy()))
-scene.set_camera(focal_point=(-18.17281532, -19.55606842, 6.92485857),
-                 position=(-360.11, -30.46, -40.44),
-                 view_up=(-0.03, 0.028, 0.89))
-window.record(scene, out_path='AF_L_recognized_bundle3.png',
-              size=(600, 600))
+scene.set_camera(
+    focal_point=(-18.17281532, -19.55606842, 6.92485857),
+    position=(-360.11, -30.46, -40.44),
+    view_up=(-0.03, 0.028, 0.89),
+)
+window.record(scene=scene, out_path="AF_L_recognized_bundle3.png", size=(600, 600))
 if interactive:
     window.show(scene)
 
@@ -291,16 +303,14 @@ if interactive:
 # Save the bundle as a trk file. Let's save the recognized bundle in the
 # common space (atlas space), in this case, MNI space.
 
-reco_af_l = StatefulTractogram(recognized_af_l, atlas_header,
-                               Space.RASMM)
+reco_af_l = StatefulTractogram(recognized_af_l, atlas_header, Space.RASMM)
 save_trk(reco_af_l, "AF_L_rec_3.trk", bbox_valid_check=False)
 
 ###############################################################################
 # Let's save the recognized bundle in the original space of the subject
 # anatomy.
 
-reco_af_l = StatefulTractogram(target[af_l_labels], target_header,
-                               Space.RASMM)
+reco_af_l = StatefulTractogram(target[af_l_labels], target_header, Space.RASMM)
 save_trk(reco_af_l, "AF_L_org_3.trk", bbox_valid_check=False)
 
 
@@ -317,9 +327,10 @@ r_recognized_af_l, r_af_l_labels = rb.refine(
     model_clust_thr=0.1,
     reduction_thr=15,
     pruning_thr=6,
-    reduction_distance='mdf',
-    pruning_distance='mdf',
-    slr=True)
+    reduction_distance="mdf",
+    pruning_distance="mdf",
+    slr=True,
+)
 
 ###############################################################################
 # let's visualize extracted refined Arcuate Fasciculus Left bundle.
@@ -329,11 +340,14 @@ interactive = False
 scene = window.Scene()
 scene.SetBackground(1, 1, 1)
 scene.add(actor.line(r_recognized_af_l.copy()))
-scene.set_camera(focal_point=(-18.17281532, -19.55606842, 6.92485857),
-                 position=(-360.11, -30.46, -40.44),
-                 view_up=(-0.03, 0.028, 0.89))
-window.record(scene, out_path='AF_L_refine_recognized_bundle.png',
-              size=(600, 600))
+scene.set_camera(
+    focal_point=(-18.17281532, -19.55606842, 6.92485857),
+    position=(-360.11, -30.46, -40.44),
+    view_up=(-0.03, 0.028, 0.89),
+)
+window.record(
+    scene=scene, out_path="AF_L_refine_recognized_bundle.png", size=(600, 600)
+)
 if interactive:
     window.show(scene)
 
@@ -347,16 +361,14 @@ if interactive:
 # Save the bundle as a trk file. Let's save the recognized bundle in the
 # common space (atlas space), in this case, MNI space.
 
-reco_af_l = StatefulTractogram(r_recognized_af_l, atlas_header,
-                               Space.RASMM)
+reco_af_l = StatefulTractogram(r_recognized_af_l, atlas_header, Space.RASMM)
 save_trk(reco_af_l, "AF_L_rec_refine.trk", bbox_valid_check=False)
 
 ###############################################################################
 # Let's save the recognized bundle in the original space of the subject
 # anatomy.
 
-reco_af_l = StatefulTractogram(target[r_af_l_labels], target_header,
-                               Space.RASMM)
+reco_af_l = StatefulTractogram(target[r_af_l_labels], target_header, Space.RASMM)
 save_trk(reco_af_l, "AF_L_org_refine.trk", bbox_valid_check=False)
 
 ###############################################################################
@@ -370,11 +382,12 @@ interactive = False
 scene = window.Scene()
 scene.SetBackground(1, 1, 1)
 scene.add(actor.line(model_cst_l))
-scene.set_camera(focal_point=(-18.17281532, -19.55606842, 6.92485857),
-                 position=(-360.11, -30.46, -40.44),
-                 view_up=(-0.03, 0.028, 0.89))
-window.record(scene, out_path='CST_L_model_bundle.png',
-              size=(600, 600))
+scene.set_camera(
+    focal_point=(-18.17281532, -19.55606842, 6.92485857),
+    position=(-360.11, -30.46, -40.44),
+    view_up=(-0.03, 0.028, 0.89),
+)
+window.record(scene=scene, out_path="CST_L_model_bundle.png", size=(600, 600))
 if interactive:
     window.show(scene)
 
@@ -383,13 +396,15 @@ if interactive:
 #
 # The Corticospinal tract model bundle
 
-recognized_cst_l, cst_l_labels = rb.recognize(model_bundle=model_cst_l,
-                                              model_clust_thr=0.1,
-                                              reduction_thr=15,
-                                              pruning_thr=7,
-                                              reduction_distance='mdf',
-                                              pruning_distance='mdf',
-                                              slr=True)
+recognized_cst_l, cst_l_labels = rb.recognize(
+    model_bundle=model_cst_l,
+    model_clust_thr=0.1,
+    reduction_thr=15,
+    pruning_thr=7,
+    reduction_distance="mdf",
+    pruning_distance="mdf",
+    slr=True,
+)
 
 ###############################################################################
 # let's visualize extracted Corticospinal tract Left bundle
@@ -399,11 +414,12 @@ interactive = False
 scene = window.Scene()
 scene.SetBackground(1, 1, 1)
 scene.add(actor.line(recognized_cst_l.copy()))
-scene.set_camera(focal_point=(-18.17281532, -19.55606842, 6.92485857),
-                 position=(-360.11, -30.46, -40.44),
-                 view_up=(-0.03, 0.028, 0.89))
-window.record(scene, out_path='CST_L_recognized_bundle.png',
-              size=(600, 600))
+scene.set_camera(
+    focal_point=(-18.17281532, -19.55606842, 6.92485857),
+    position=(-360.11, -30.46, -40.44),
+    view_up=(-0.03, 0.028, 0.89),
+)
+window.record(scene=scene, out_path="CST_L_recognized_bundle.png", size=(600, 600))
 if interactive:
     window.show(scene)
 
@@ -417,28 +433,28 @@ if interactive:
 # Save the bundle as a trk file. Let's save the recognized bundle in the
 # common space (atlas space), in this case, MNI space.
 
-reco_cst_l = StatefulTractogram(recognized_cst_l, atlas_header,
-                                Space.RASMM)
+reco_cst_l = StatefulTractogram(recognized_cst_l, atlas_header, Space.RASMM)
 save_trk(reco_cst_l, "CST_L_rec_1.trk", bbox_valid_check=False)
 
 ###############################################################################
 # Let's save the recognized bundle in the original space of the subject
 # anatomy.
 
-reco_cst_l = StatefulTractogram(target[cst_l_labels], target_header,
-                                Space.RASMM)
+reco_cst_l = StatefulTractogram(target[cst_l_labels], target_header, Space.RASMM)
 save_trk(reco_cst_l, "CST_L_org_1.trk", bbox_valid_check=False)
 
 ###############################################################################
 # Now, let's increase the reduction_thr and pruning_thr values.
 
-recognized_cst_l, cst_l_labels = rb.recognize(model_bundle=model_cst_l,
-                                              model_clust_thr=0.1,
-                                              reduction_thr=20,
-                                              pruning_thr=10,
-                                              reduction_distance='mdf',
-                                              pruning_distance='mdf',
-                                              slr=True)
+recognized_cst_l, cst_l_labels = rb.recognize(
+    model_bundle=model_cst_l,
+    model_clust_thr=0.1,
+    reduction_thr=20,
+    pruning_thr=10,
+    reduction_distance="mdf",
+    pruning_distance="mdf",
+    slr=True,
+)
 
 ###############################################################################
 # let's visualize extracted Corticospinal tract Left bundle.
@@ -448,11 +464,12 @@ interactive = False
 scene = window.Scene()
 scene.SetBackground(1, 1, 1)
 scene.add(actor.line(recognized_cst_l.copy()))
-scene.set_camera(focal_point=(-18.17281532, -19.55606842, 6.92485857),
-                 position=(-360.11, -30.46, -40.44),
-                 view_up=(-0.03, 0.028, 0.89))
-window.record(scene, out_path='CST_L_recognized_bundle2.png',
-              size=(600, 600))
+scene.set_camera(
+    focal_point=(-18.17281532, -19.55606842, 6.92485857),
+    position=(-360.11, -30.46, -40.44),
+    view_up=(-0.03, 0.028, 0.89),
+)
+window.record(scene=scene, out_path="CST_L_recognized_bundle2.png", size=(600, 600))
 if interactive:
     window.show(scene)
 
@@ -465,28 +482,28 @@ if interactive:
 # Save the bundle as a trk file. Let's save the recognized bundle in the
 # common space (atlas space), in this case, MNI space.
 
-reco_cst_l = StatefulTractogram(recognized_cst_l, atlas_header,
-                                Space.RASMM)
+reco_cst_l = StatefulTractogram(recognized_cst_l, atlas_header, Space.RASMM)
 save_trk(reco_cst_l, "CST_L_rec_2.trk", bbox_valid_check=False)
 
 ###############################################################################
 # Let's save the recognized bundle in the original space of the subject
 # anatomy.
 
-reco_cst_l = StatefulTractogram(target[cst_l_labels], target_header,
-                                Space.RASMM)
+reco_cst_l = StatefulTractogram(target[cst_l_labels], target_header, Space.RASMM)
 save_trk(reco_cst_l, "CST_L_org_2.trk", bbox_valid_check=False)
 
 ###############################################################################
 # Now, let's increase the reduction_thr and pruning_thr values further.
 
-recognized_cst_l, cst_l_labels = rb.recognize(model_bundle=model_cst_l,
-                                              model_clust_thr=0.1,
-                                              reduction_thr=25,
-                                              pruning_thr=12,
-                                              reduction_distance='mdf',
-                                              pruning_distance='mdf',
-                                              slr=True)
+recognized_cst_l, cst_l_labels = rb.recognize(
+    model_bundle=model_cst_l,
+    model_clust_thr=0.1,
+    reduction_thr=25,
+    pruning_thr=12,
+    reduction_distance="mdf",
+    pruning_distance="mdf",
+    slr=True,
+)
 
 ###############################################################################
 # let's visualize extracted Corticospinal tract Left bundle.
@@ -496,11 +513,12 @@ interactive = False
 scene = window.Scene()
 scene.SetBackground(1, 1, 1)
 scene.add(actor.line(recognized_cst_l.copy()))
-scene.set_camera(focal_point=(-18.17281532, -19.55606842, 6.92485857),
-                 position=(-360.11, -30.46, -40.44),
-                 view_up=(-0.03, 0.028, 0.89))
-window.record(scene, out_path='CST_L_recognized_bundle3.png',
-              size=(600, 600))
+scene.set_camera(
+    focal_point=(-18.17281532, -19.55606842, 6.92485857),
+    position=(-360.11, -30.46, -40.44),
+    view_up=(-0.03, 0.028, 0.89),
+)
+window.record(scene=scene, out_path="CST_L_recognized_bundle3.png", size=(600, 600))
 if interactive:
     window.show(scene)
 
@@ -515,16 +533,14 @@ if interactive:
 # Save the bundle as a trk file. Let's save the recognized bundle in the
 # common space (atlas space), in this case, MNI space.
 
-reco_cst_l = StatefulTractogram(recognized_cst_l, atlas_header,
-                                Space.RASMM)
+reco_cst_l = StatefulTractogram(recognized_cst_l, atlas_header, Space.RASMM)
 save_trk(reco_cst_l, "CST_L_rec_3.trk", bbox_valid_check=False)
 
 ###############################################################################
 # Let's save the recognized bundle in the original space of the subject
 # anatomy.
 
-reco_cst_l = StatefulTractogram(target[cst_l_labels], target_header,
-                                Space.RASMM)
+reco_cst_l = StatefulTractogram(target[cst_l_labels], target_header, Space.RASMM)
 save_trk(reco_cst_l, "CST_L_org_3.trk", bbox_valid_check=False)
 
 ###############################################################################
@@ -535,14 +551,15 @@ save_trk(reco_cst_l, "CST_L_org_3.trk", bbox_valid_check=False)
 # smaller values in the auto-calibrated RecoBundles (refinement) step.
 
 r_recognized_cst_l, r_cst_l_labels = rb.refine(
-                                     model_bundle=model_cst_l,
-                                     pruned_streamlines=recognized_cst_l,
-                                     model_clust_thr=0.1,
-                                     reduction_thr=15,
-                                     pruning_thr=6,
-                                     reduction_distance='mdf',
-                                     pruning_distance='mdf',
-                                     slr=True)
+    model_bundle=model_cst_l,
+    pruned_streamlines=recognized_cst_l,
+    model_clust_thr=0.1,
+    reduction_thr=15,
+    pruning_thr=6,
+    reduction_distance="mdf",
+    pruning_distance="mdf",
+    slr=True,
+)
 
 ###############################################################################
 # let's visualize extracted refined Corticospinal tract Left bundle.
@@ -552,11 +569,14 @@ interactive = False
 scene = window.Scene()
 scene.SetBackground(1, 1, 1)
 scene.add(actor.line(r_recognized_cst_l.copy()))
-scene.set_camera(focal_point=(-18.17281532, -19.55606842, 6.92485857),
-                 position=(-360.11, -30.46, -40.44),
-                 view_up=(-0.03, 0.028, 0.89))
-window.record(scene, out_path='CST_L_refine_recognized_bundle.png',
-              size=(600, 600))
+scene.set_camera(
+    focal_point=(-18.17281532, -19.55606842, 6.92485857),
+    position=(-360.11, -30.46, -40.44),
+    view_up=(-0.03, 0.028, 0.89),
+)
+window.record(
+    scene=scene, out_path="CST_L_refine_recognized_bundle.png", size=(600, 600)
+)
 if interactive:
     window.show(scene)
 
@@ -569,16 +589,14 @@ if interactive:
 # Save the bundle as a trk file. Let's save the recognized bundle in the
 # common space (atlas space), in this case, MNI space.
 
-reco_cst_l = StatefulTractogram(r_recognized_cst_l, atlas_header,
-                                Space.RASMM)
+reco_cst_l = StatefulTractogram(r_recognized_cst_l, atlas_header, Space.RASMM)
 save_trk(reco_cst_l, "CST_L_rec_refine.trk", bbox_valid_check=False)
 
 ###############################################################################
 # Let's save the recognized bundle in the original space of the subject
 # anatomy.
 
-reco_cst_l = StatefulTractogram(target[r_cst_l_labels], target_header,
-                                Space.RASMM)
+reco_cst_l = StatefulTractogram(target[r_cst_l_labels], target_header, Space.RASMM)
 save_trk(reco_cst_l, "CST_L_org_refine.trk", bbox_valid_check=False)
 
 ###############################################################################
@@ -589,12 +607,5 @@ save_trk(reco_cst_l, "CST_L_org_refine.trk", bbox_valid_check=False)
 # References
 # ----------
 #
-# .. [Garyfallidis17] Garyfallidis et al. Recognition of white matter
-#         bundles using local and global streamline-based registration
-#         and clustering, Neuroimage, 2017.
+# .. footbibliography::
 #
-# .. [Chandio2020] Chandio, B.Q., Risacher, S.L., Pestilli, F.,
-#         Bullock, D., Yeh, FC., Koudoro, S., Rokem, A., Harezlak, J., and
-#         Garyfallidis, E. Bundle analytics, a computational framework for
-#         investigating the shapes and profiles of brain pathways across
-#         populations. Sci Rep 10, 17149 (2020)
