@@ -21,33 +21,32 @@ data and fitting a Constrained Spherical Deconvolution (CSD) model.
 """
 
 from dipy.core.gradients import gradient_table
-from dipy.data import get_fnames, small_sphere, default_sphere
+from dipy.data import default_sphere, get_fnames, small_sphere
 from dipy.direction import ProbabilisticDirectionGetter, peaks_from_model
 from dipy.io.gradients import read_bvals_bvecs
 from dipy.io.image import load_nifti, load_nifti_data
 from dipy.io.stateful_tractogram import Space, StatefulTractogram
 from dipy.io.streamline import save_trk
-from dipy.reconst.csdeconv import (ConstrainedSphericalDeconvModel,
-                                   auto_response_ssst)
+from dipy.reconst.csdeconv import ConstrainedSphericalDeconvModel, auto_response_ssst
 from dipy.reconst.shm import CsaOdfModel
 from dipy.tracking import utils
 from dipy.tracking.local_tracking import LocalTracking
-from dipy.tracking.streamline import Streamlines
 from dipy.tracking.stopping_criterion import ThresholdStoppingCriterion
-from dipy.viz import window, actor, colormap, has_fury
+from dipy.tracking.streamline import Streamlines
+from dipy.viz import actor, colormap, has_fury, window
 
 # Enables/disables interactive visualization
 interactive = False
 
-hardi_fname, hardi_bval_fname, hardi_bvec_fname = get_fnames('stanford_hardi')
-label_fname = get_fnames('stanford_labels')
+hardi_fname, hardi_bval_fname, hardi_bvec_fname = get_fnames(name="stanford_hardi")
+label_fname = get_fnames(name="stanford_labels")
 
 data, affine, hardi_img = load_nifti(hardi_fname, return_img=True)
 labels = load_nifti_data(label_fname)
 bvals, bvecs = read_bvals_bvecs(hardi_bval_fname, hardi_bvec_fname)
-gtab = gradient_table(bvals, bvecs)
+gtab = gradient_table(bvals, bvecs=bvecs)
 
-seed_mask = (labels == 2)
+seed_mask = labels == 2
 white_matter = (labels == 1) | (labels == 2)
 seeds = utils.seeds_from_mask(seed_mask, affine, density=1)
 
@@ -60,7 +59,7 @@ csd_fit = csd_model.fit(data, mask=white_matter)
 
 csa_model = CsaOdfModel(gtab, sh_order_max=6)
 gfa = csa_model.fit(data, mask=white_matter).gfa
-stopping_criterion = ThresholdStoppingCriterion(gfa, .25)
+stopping_criterion = ThresholdStoppingCriterion(gfa, 0.25)
 
 ###############################################################################
 # The Fiber Orientation Distribution (FOD) of the CSD model estimates the
@@ -75,19 +74,22 @@ stopping_criterion = ThresholdStoppingCriterion(gfa, .25)
 
 fod = csd_fit.odf(small_sphere)
 pmf = fod.clip(min=0)
-prob_dg = ProbabilisticDirectionGetter.from_pmf(pmf, max_angle=30.,
-                                                sphere=small_sphere)
-streamline_generator = LocalTracking(prob_dg, stopping_criterion, seeds,
-                                     affine, step_size=.5)
+prob_dg = ProbabilisticDirectionGetter.from_pmf(
+    pmf, max_angle=30.0, sphere=small_sphere
+)
+streamline_generator = LocalTracking(
+    prob_dg, stopping_criterion, seeds, affine, step_size=0.5
+)
 streamlines = Streamlines(streamline_generator)
 sft = StatefulTractogram(streamlines, hardi_img, Space.RASMM)
 save_trk(sft, "tractogram_probabilistic_dg_pmf.trk")
 
 if has_fury:
     scene = window.Scene()
-    scene.add(actor.line(streamlines, colormap.line_colors(streamlines)))
-    window.record(scene, out_path='tractogram_probabilistic_dg_pmf.png',
-                  size=(800, 800))
+    scene.add(actor.line(streamlines, colors=colormap.line_colors(streamlines)))
+    window.record(
+        scene=scene, out_path="tractogram_probabilistic_dg_pmf.png", size=(800, 800)
+    )
     if interactive:
         window.show(scene)
 
@@ -109,21 +111,22 @@ if has_fury:
 # use a larger sphere, like ``default_sphere`` which has 362 directions on the
 # hemisphere, without having to worry about memory limitations.
 
-prob_dg = ProbabilisticDirectionGetter.from_shcoeff(csd_fit.shm_coeff,
-                                                    max_angle=30.,
-                                                    sphere=default_sphere,
-                                                    sh_to_pmf=True)
-streamline_generator = LocalTracking(prob_dg, stopping_criterion, seeds,
-                                     affine, step_size=.5)
+prob_dg = ProbabilisticDirectionGetter.from_shcoeff(
+    csd_fit.shm_coeff, max_angle=30.0, sphere=default_sphere, sh_to_pmf=True
+)
+streamline_generator = LocalTracking(
+    prob_dg, stopping_criterion, seeds, affine, step_size=0.5
+)
 streamlines = Streamlines(streamline_generator)
 sft = StatefulTractogram(streamlines, hardi_img, Space.RASMM)
 save_trk(sft, "tractogram_probabilistic_dg_sh.trk")
 
 if has_fury:
     scene = window.Scene()
-    scene.add(actor.line(streamlines, colormap.line_colors(streamlines)))
-    window.record(scene, out_path='tractogram_probabilistic_dg_sh.png',
-                  size=(800, 800))
+    scene.add(actor.line(streamlines, colors=colormap.line_colors(streamlines)))
+    window.record(
+        scene=scene, out_path="tractogram_probabilistic_dg_sh.png", size=(800, 800)
+    )
     if interactive:
         window.show(scene)
 
@@ -139,25 +142,35 @@ if has_fury:
 # of any model to the spherical harmonic basis using the ``peaks_from_model``
 # function.
 
-peaks = peaks_from_model(csd_model, data, default_sphere, .5, 25,
-                         mask=white_matter, return_sh=True, parallel=True,
-                         num_processes=2)
+peaks = peaks_from_model(
+    csd_model,
+    data,
+    default_sphere,
+    0.5,
+    25,
+    mask=white_matter,
+    return_sh=True,
+    parallel=True,
+    num_processes=2,
+)
 fod_coeff = peaks.shm_coeff
 
-prob_dg = ProbabilisticDirectionGetter.from_shcoeff(fod_coeff, max_angle=30.,
-                                                    sphere=default_sphere,
-                                                    sh_to_pmf=True)
-streamline_generator = LocalTracking(prob_dg, stopping_criterion, seeds,
-                                     affine, step_size=.5)
+prob_dg = ProbabilisticDirectionGetter.from_shcoeff(
+    fod_coeff, max_angle=30.0, sphere=default_sphere, sh_to_pmf=True
+)
+streamline_generator = LocalTracking(
+    prob_dg, stopping_criterion, seeds, affine, step_size=0.5
+)
 streamlines = Streamlines(streamline_generator)
 sft = StatefulTractogram(streamlines, hardi_img, Space.RASMM)
 save_trk(sft, "tractogram_probabilistic_dg_sh_pfm.trk")
 
 if has_fury:
     scene = window.Scene()
-    scene.add(actor.line(streamlines, colormap.line_colors(streamlines)))
-    window.record(scene, out_path='tractogram_probabilistic_dg_sh_pfm.png',
-                  size=(800, 800))
+    scene.add(actor.line(streamlines, colors=colormap.line_colors(streamlines)))
+    window.record(
+        scene=scene, out_path="tractogram_probabilistic_dg_sh_pfm.png", size=(800, 800)
+    )
     if interactive:
         window.show(scene)
 
